"""
分数除法算子 - 分数÷分数
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionType


class FractionDivideCalculator(FractionCalculator):
    """分数除法算子"""
    
    def __init__(self):
        super().__init__("分数除法", "除以分数等于乘以倒数", priority=5)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配分数除法问题，支持分数÷分数、分数÷整数、整数÷分数"""
        if problem.operation != OperationType.DIVIDE:
            return {"matched": False, "score": 0.0, "reason": "不是除法"}
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        dividend = problem.operands[0]
        divisor = problem.operands[1]
        # 被除数、除数均可为分数或整数
        is_dividend_fraction = (
            getattr(dividend, 'fraction_type', None) in [FractionType.PROPER, FractionType.IMPROPER, FractionType.MIXED]
            or dividend.fraction.denominator != 1
        )
        is_dividend_integer = dividend.fraction.denominator == 1
        is_divisor_fraction = (
            getattr(divisor, 'fraction_type', None) in [FractionType.PROPER, FractionType.IMPROPER, FractionType.MIXED]
            or divisor.fraction.denominator != 1
        )
        is_divisor_integer = divisor.fraction.denominator == 1
        # 只要有一方是分数即可
        if (is_dividend_fraction or is_divisor_fraction):
            return {"matched": True, "score": 1.0, "reason": "支持整数÷分数、分数÷整数、分数÷分数"}
        return {"matched": False, "score": 0.0, "reason": "不支持的除法类型"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行分数除法，支持整数÷分数、分数÷整数、分数÷分数"""
        try:
            dividend_op, divisor_op = problem.operands  # 被除数，除数
            steps = []
            # 步骤1：检查除数不为0
            if divisor_op.fraction.numerator == 0:
                return FractionResult(
                    success=False,
                    error="除数不能为0"
                )
            steps.append(FractionStep(
                description=f"检查除数{divisor_op.fraction}不为0",
                operation="检查除数非零",
                result="除数非零，可以计算"
            ))
            # 步骤2：将带分数转为假分数，整数直接转为分母为1的分数
            def to_fraction(op):
                if getattr(op, 'is_mixed', False):
                    return Fraction(op.whole_part * op.fraction.denominator + op.fraction.numerator, op.fraction.denominator)
                return op.fraction
            dividend = to_fraction(dividend_op)
            divisor = to_fraction(divisor_op)
            if getattr(dividend_op, 'is_mixed', False):
                steps.append(FractionStep(
                    description=f"被除数转假分数：{dividend_op.raw} → {dividend}",
                    operation="转为假分数",
                    result=str(dividend)
                ))
            if getattr(divisor_op, 'is_mixed', False):
                steps.append(FractionStep(
                    description=f"除数转假分数：{divisor_op.raw} → {divisor}",
                    operation="转为假分数",
                    result=str(divisor)
                ))
            # 步骤3：将除法转为乘法（取倒数）
            reciprocal = Fraction(divisor.denominator, divisor.numerator)
            steps.append(FractionStep(
                description=f"除以分数等于乘以倒数：{dividend} ÷ {divisor} = {dividend} × {reciprocal}",
                operation="取倒数并转换为乘法",
                result=f"{dividend} × {reciprocal}",
                formula="a/b ÷ c/d = a/b × d/c"
            ))
            # 步骤4：执行乘法（交叉约分）
            from math import gcd
            num1, den1 = dividend.numerator, dividend.denominator
            num2, den2 = reciprocal.numerator, reciprocal.denominator
            # 交叉约分
            gcd1 = gcd(abs(num1), abs(den2))
            if gcd1 > 1:
                num1 //= gcd1
                den2 //= gcd1
                steps.append(FractionStep(
                    description=f"交叉约分：{dividend.numerator}与{reciprocal.denominator}的公因子{gcd1}",
                    operation="交叉约分",
                    result=f"约分后：{num1}，{den2}"
                ))
            gcd2 = gcd(abs(num2), abs(den1))
            if gcd2 > 1:
                num2 //= gcd2
                den1 //= gcd2
                steps.append(FractionStep(
                    description=f"交叉约分：{reciprocal.numerator}与{dividend.denominator}的公因子{gcd2}",
                    operation="交叉约分",
                    result=f"约分后：{num2}，{den1}"
                ))
            # 相乘得结果
            result_num = num1 * num2
            result_den = den1 * den2
            result_fraction = Fraction(result_num, result_den)
            steps.append(FractionStep(
                description=f"相乘：({num1}×{num2})/({den1}×{den2}) = {result_fraction}",
                operation="分子分母分别相乘",
                result=str(result_fraction)
            ))
            # 步骤5：最终约分和格式化
            final_result = result_fraction
            if result_fraction.denominator != 1:
                final_gcd = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if final_gcd > 1:
                    final_result = Fraction(result_fraction.numerator // final_gcd,
                                          result_fraction.denominator // final_gcd)
                    steps.append(FractionStep(
                        description=f"最终约分：{result_fraction} → {final_result}",
                        operation="取得最简分数",
                        result=str(final_result)
                    ))
            # 转为带分数（如需要）
            display_result = final_result
            if (problem.display_mixed and 
                abs(final_result.numerator) >= final_result.denominator and 
                final_result.denominator != 1):
                whole = final_result.numerator // final_result.denominator
                remainder = abs(final_result.numerator) % final_result.denominator
                if remainder == 0:
                    display_result = str(whole)
                else:
                    display_result = f"{whole} {remainder}/{final_result.denominator}"
                steps.append(FractionStep(
                    description=f"转为带分数：{final_result} = {display_result}",
                    operation="转为带分数",
                    result=str(display_result)
                ))
            return FractionResult(
                success=True,
                result=display_result,
                steps=steps,
                step_count=len(steps),
                formula=f"分数除法：{dividend} ÷ {divisor} = {display_result}",
                validation=True
            )
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"分数除法计算失败: {str(e)}"
            )



"""
首数估商法实现
在长除法中，只看被除数和除数的首位（或前两位）来快速估算商的第一位
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class FirstDigitEstimateDivision(MathCalculator):
    """首数估商法算法"""
    
    def __init__(self):
        super().__init__("首数估商法", "看首位，估商数", priority=4)
    
    def _get_first_digits(self, num, count: int = 2) -> int:
        """获取数字的前几位"""
        str_num = str(int(num))
        if len(str_num) <= count:
            return int(str_num)
        return int(str_num[:count])
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：多位数除法，适合首位估算"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            
            if not (isinstance(dividend, (int, float)) and isinstance(divisor, (int, float))):
                return False
            
            if divisor == 0:
                return False
            
            # 被除数必须是多位数（至少3位）
            if dividend < 100:
                return False
            
            # 除数应该是1-2位数，便于心算
            if divisor >= 100 or divisor < 2:
                return False
            
            # 商应该是多位数，这样首数估商才有意义
            quotient = dividend / divisor
            if quotient < 10:
                return False
            
            return True
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建首数估商法步骤"""
        numbers = formula.get_numbers()
        dividend = numbers[0].get_numeric_value()
        divisor = numbers[1].get_numeric_value()
        
        # 确保是整数
        dividend_int = int(dividend)
        divisor_int = int(divisor)
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend_int} ÷ {divisor_int} 使用首数估商法",
            operation="识别多位除法",
            result="看首位，估商数"
        ))
        
        # 分析被除数和除数的位数
        dividend_str = str(dividend_int)
        divisor_str = str(divisor_int)
        dividend_digits = len(dividend_str)
        divisor_digits = len(divisor_str)
        
        steps.append(CalculationStep(
            description=f"被除数{dividend_int}是{dividend_digits}位数，除数{divisor_int}是{divisor_digits}位数",
            operation="分析位数",
            result=f"商大约是{dividend_digits - divisor_digits + 1}位数"
        ))
        
        # 首数估算
        if divisor_digits == 1:
            # 除数是一位数
            dividend_first = int(dividend_str[0])
            if dividend_first >= divisor_int:
                first_quotient = dividend_first // divisor_int
                steps.append(CalculationStep(
                    description=f"首位估商：{dividend_first} ÷ {divisor_int} ≈ {first_quotient}",
                    operation="首位估算",
                    result=f"商的首位大约是{first_quotient}"
                ))
            else:
                # 需要看前两位
                dividend_first_two = int(dividend_str[:2])
                first_quotient = dividend_first_two // divisor_int
                steps.append(CalculationStep(
                    description=f"首位不够，看前两位：{dividend_first_two} ÷ {divisor_int} ≈ {first_quotient}",
                    operation="前两位估算",
                    result=f"商的首位大约是{first_quotient}"
                ))
        else:
            # 除数是两位数
            dividend_first_n = self._get_first_digits(dividend_int, divisor_digits + 1)
            divisor_first_n = self._get_first_digits(divisor_int, divisor_digits)
            
            first_quotient = dividend_first_n // divisor_first_n
            
            steps.append(CalculationStep(
                description=f"首数估商：{dividend_first_n} ÷ {divisor_first_n} ≈ {first_quotient}",
                operation="首数估算",
                result=f"商的首位大约是{first_quotient}"
            ))
        
        # 验证估算并调整
        estimated_product = first_quotient * divisor_int
        
        # 根据位数确定估算商的量级
        expected_digits = dividend_digits - divisor_digits + 1
        if expected_digits > 1:
            estimated_quotient = first_quotient * (10 ** (expected_digits - 1))
            estimated_product = estimated_quotient * divisor_int
            
            steps.append(CalculationStep(
                description=f"估算完整商：约{estimated_quotient}，验证：{estimated_quotient} × {divisor_int} = {estimated_product}",
                operation="估算验证",
                result=f"估算商接近{estimated_quotient}"
            ))
        
        # 计算精确结果
        actual_quotient = dividend_int / divisor_int
        
        if isinstance(actual_quotient, float) and actual_quotient.is_integer():
            actual_quotient = int(actual_quotient)
        
        # 比较估算和实际结果
        if isinstance(actual_quotient, int):
            actual_str = str(actual_quotient)
            estimated_first = str(first_quotient)[0]
            actual_first = actual_str[0]
            
            if estimated_first == actual_first:
                steps.append(CalculationStep(
                    description=f"估算准确！实际商：{actual_quotient}，首位确实是{actual_first}",
                    operation="估算成功",
                    result="✓ 首位估算正确"
                ))
            else:
                steps.append(CalculationStep(
                    description=f"估算偏差：实际商{actual_quotient}，首位是{actual_first}而不是{estimated_first}",
                    operation="估算调整",
                    result=f"需要微调估算"
                ))
        
        steps.append(CalculationStep(
            description=f"精确结果：{dividend_int} ÷ {divisor_int} = {actual_quotient}",
            operation="精确计算",
            result=actual_quotient,
            formula="首数估商法：先估首位，后精算"
        ))
        
        # 提供心算技巧
        if divisor_int <= 12:
            steps.append(CalculationStep(
                description=f"心算技巧：熟记{divisor_int}的倍数表，快速确定商的范围",
                operation="心算提示",
                result=f"{divisor_int}的倍数：{divisor_int}×10={divisor_int*10}, {divisor_int}×100={divisor_int*100}"
            ))
        
        return steps


"""
几十一乘几十一实现
形如21×31的算法，公式(10a+1)(10b+1)=100ab+10(a+b)+1
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class TensOneMultiplication(MathCalculator):
    """几十一乘几十一算法"""
    
    def __init__(self):
        super().__init__("几十一乘法", "几十一乘几十一，前乘百和乘十再加一", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：形如21×31，个位数都是1"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 检查是否都是两位数且个位数都是1
            if 10 <= a <= 99 and 10 <= b <= 99:
                return a % 10 == 1 and b % 10 == 1
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建几十一乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 提取十位数字
        tens_a = a // 10
        tens_b = b // 10
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"识别几十一乘法：{a} 和 {b}",
            operation="识别模式",
            result=f"都是个位为1的两位数：{tens_a}1 × {tens_b}1"
        ))
        
        steps.append(CalculationStep(
            description=f"提取十位数：{tens_a} 和 {tens_b}",
            operation="分解数字",
            result=f"a={tens_a}, b={tens_b}"
        ))
        
        # 计算各部分
        hundreds_part = tens_a * tens_b * 100
        tens_part = (tens_a + tens_b) * 10
        ones_part = 1
        
        steps.append(CalculationStep(
            description=f"前乘百：{tens_a} × {tens_b} × 100 = {hundreds_part}",
            operation="计算百位部分",
            result=hundreds_part
        ))
        
        steps.append(CalculationStep(
            description=f"和乘十：({tens_a} + {tens_b}) × 10 = {tens_part}",
            operation="计算十位部分",
            result=tens_part
        ))
        
        steps.append(CalculationStep(
            description=f"再加一：+ 1",
            operation="加个位数",
            result=ones_part
        ))
        
        # 最终结果
        final_result = hundreds_part + tens_part + ones_part
        
        steps.append(CalculationStep(
            description=f"最终结果：{hundreds_part} + {tens_part} + {ones_part} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="(10a+1)(10b+1) = 100ab + 10(a+b) + 1"
        ))
        
        return steps


"""
混合运算基础类型定义
"""

from abc import ABC, abstractmethod
from typing import Dict, Any, List, Optional, Union
from dataclasses import dataclass
from enum import Enum
from decimal import Decimal


class OperationType(Enum):
    """运算类型"""
    ADD = "+"
    SUBTRACT = "-"
    MULTIPLY = "*"
    DIVIDE = "/"
    PARENTHESES = "()"
    EXPRESSION = "expression"


class NumberType(Enum):
    """数值类型"""
    INTEGER = "integer"
    DECIMAL = "decimal"
    FRACTION = "fraction"


@dataclass
class MixedOperand:
    """混合运算操作数"""
    value: Union[int, float, Decimal]
    original: str  # 原始输入
    number_type: NumberType
    position: int = 0  # 在表达式中的位置


@dataclass
class MixedStep:
    """混合运算步骤"""
    description: str
    operation: str
    inputs: List[str] = None
    result: str = ""
    formula: str = ""
    validation: bool = True
    meta: Dict[str, Any] = None
    
    def __post_init__(self):
        if self.inputs is None:
            self.inputs = []
        if self.meta is None:
            self.meta = {}
    def __str__(self):
        return f"{self.description}: 结果： {self.result}"
    def to_dict(self) -> Dict[str, Any]:
        result = self.result
        return {
            "description": self.description,
            "operation": self.operation,
            "result": result,
            "formula": self.formula
        }


@dataclass
class MixedProblem:
    """混合运算问题"""
    expression: str
    operands: List[MixedOperand]
    operations: List[str]
    has_parentheses: bool = False
    complexity_level: int = 1  # 复杂度等级 1-5
    special_numbers: List[int] = None  # 特殊数字(0,1)的位置
    
    def __post_init__(self):
        if self.special_numbers is None:
            self.special_numbers = []


@dataclass
class MixedResult:
    """混合运算结果"""
    success: bool
    result: Union[str, int, float, Decimal] = None
    steps: List[MixedStep] = None
    step_count: int = 0
    formula: str = ""
    validation: bool = True
    error: str = ""
    technique_used: str = ""  # 使用的技巧
    
    def __post_init__(self):
        if self.steps is None:
            self.steps = []
        self.step_count = len(self.steps)


class MixedUtils:
    """混合运算工具类"""
    
    @staticmethod
    def is_special_number(num: Union[int, float]) -> bool:
        """判断是否为特殊数字"""
        return num in [0, 1, -1, 10, 100, 0.1, 0.5]
    
    @staticmethod
    def can_use_complement(a: Union[int, float], b: Union[int, float]) -> bool:
        """判断是否可以使用补数技巧"""
        if isinstance(a, (int, float)) and isinstance(b, (int, float)):
            # 检查是否能凑成整十、整百
            sum_val = a + b
            diff_val = abs(a - b)
            return (sum_val % 10 == 0 or sum_val % 100 == 0 or 
                    diff_val % 10 == 0 or diff_val % 100 == 0)
        return False
    
    @staticmethod
    def get_complement_strategy(a: Union[int, float], b: Union[int, float], op: str) -> Optional[Dict[str, Any]]:
        """获取补数策略"""
        if op == "+":
            if (a + b) % 10 == 0:
                return {"type": "sum_to_ten", "target": a + b}
            elif (a + b) % 100 == 0:
                return {"type": "sum_to_hundred", "target": a + b}
        elif op == "-":
            if abs(a - b) % 10 == 0:
                return {"type": "diff_to_ten", "target": abs(a - b)}
        return None
    
    @staticmethod
    def can_use_distribution(expr_parts: List[str]) -> bool:
        """判断是否可以使用分配律"""
        # 检查是否有 a*(b+c) 或 a*(b-c) 的模式
        for i, part in enumerate(expr_parts):
            if '*' in part and '(' in part:
                return True
        return False


class MixedCalculator(ABC):
    """混合运算算子基类"""
    
    def __init__(self, name: str, description: str, priority: int = 10):
        self.name = name
        self.description = description
        self.priority = priority
    
    @abstractmethod
    def is_match_pattern(self, problem: MixedProblem) -> Dict[str, Any]:
        """
        判断是否匹配当前算子
        
        Returns:
            {
                "matched": bool,
                "score": float,  # 匹配分数 0-1
                "reason": str    # 匹配原因
            }
        """
        pass
    
    @abstractmethod
    def solve(self, problem: MixedProblem) -> MixedResult:
        """
        执行计算
        
        Args:
            problem: 混合运算问题
            
        Returns:
            MixedResult: 计算结果
        """
        pass



"""
近整百减法实现
处理接近整百数的减法，如207-198=9
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class NearHundredSubtraction(MathCalculator):
    """近整百减法算法"""
    
    def __init__(self):
        super().__init__("近整百减法", "处理接近整百数的减法", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：被减数和减数都接近整百数"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, int) and isinstance(subtrahend, int)):
                return False
            
            # 三位数且都接近整百数
            if not (100 <= minuend <= 999 and 100 <= subtrahend <= 999):
                return False
            
            # 检查是否都接近整百数（与最近整百数的差距小于等于10）
            minuend_hundred = round(minuend / 100) * 100
            subtrahend_hundred = round(subtrahend / 100) * 100
            
            minuend_diff = abs(minuend - minuend_hundred)
            subtrahend_diff = abs(subtrahend - subtrahend_hundred)
            
            # 都接近整百数
            return minuend_diff <= 10 and subtrahend_diff <= 10
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建近整百减法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用近整百减法",
            operation="识别近整百数",
            result="被减数和减数都接近整百数"
        ))
        
        # 找到最近的整百数
        minuend_hundred = round(minuend / 100) * 100
        subtrahend_hundred = round(subtrahend / 100) * 100
        
        # 计算与整百数的差值
        minuend_diff = minuend - minuend_hundred
        subtrahend_diff = subtrahend - subtrahend_hundred
        
        steps.append(CalculationStep(
            description=f"分解为整百数部分：{minuend} = {minuend_hundred} + ({minuend_diff}), {subtrahend} = {subtrahend_hundred} + ({subtrahend_diff})",
            operation="分解整百数",
            result=f"整百数部分：{minuend_hundred} - {subtrahend_hundred}"
        ))
        
        # 先计算整百数部分
        hundred_result = minuend_hundred - subtrahend_hundred
        steps.append(CalculationStep(
            description=f"整百数相减：{minuend_hundred} - {subtrahend_hundred} = {hundred_result}",
            operation="计算整百数差",
            result=hundred_result
        ))
        
        # 计算余数部分
        remainder_diff = minuend_diff - subtrahend_diff
        steps.append(CalculationStep(
            description=f"余数相减：({minuend_diff}) - ({subtrahend_diff}) = {remainder_diff}",
            operation="计算余数差",
            result=remainder_diff
        ))
        
        # 最终结果
        final_result = hundred_result + remainder_diff
        steps.append(CalculationStep(
            description=f"组合结果：{hundred_result} + ({remainder_diff}) = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="近整百减法：(100a+x) - (100b+y) = 100(a-b) + (x-y)"
        ))
        
        return steps


"""
合十数乘两位重复数实现
合十数对：个位数字相加等于10的一对数，如73和77（3+7=10）
重复数：两位数中十位和个位相同，如11, 22, 33...99
规律：合十数对中的一个数乘以重复数 = 个位数字乘积 × 重复数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class ComplementTenMultiplyRepeated(MathCalculator):
    """合十数乘两位重复数算法"""
    
    def __init__(self):
        super().__init__("合十数乘重复数", "合十数乘两位重复数，积的规律", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：合十数乘两位重复数"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 检查是否都是两位数
            if not (10 <= a <= 99 and 10 <= b <= 99):
                return False
            
            # 检查是否有重复数（11, 22, 33...99）
            def is_repeated_number(num):
                str_num = str(num)
                return len(str_num) == 2 and str_num[0] == str_num[1]
            
            # 检查是否是合十数对
            def is_complement(num1):
                ones1, tens1 = num1 % 10, num1 // 10
                return ones1 + tens1 == 10

            # 如果a是重复数，b是合十数
            if is_repeated_number(a):
                return is_complement(b)
            # 如果b是重复数，a是合十数
            if is_repeated_number(b):
                return is_complement(a)
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建合十数乘重复数步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是重复数，哪个是合十数
        def is_repeated_number(num):
            str_num = str(num)
            return len(str_num) == 2 and str_num[0] == str_num[1]
        
        if is_repeated_number(a):
            repeated_num = a
            complement_num = b
        else:
            repeated_num = b
            complement_num = a
        
        # 分解数字：合十数 = A×10 + B，重复数 = C×10 + C
        A = complement_num // 10
        B = complement_num % 10
        C = repeated_num // 10  # 重复数的数字
        
        # 计算合十数对
        complement_pair_ones = 10 - B
        complement_pair = A * 10 + complement_pair_ones
        
        steps = []
        
        # 步骤1：识别模式
        steps.append(CalculationStep(
            description=f"识别合十数乘重复数：{complement_num} × {repeated_num}",
            operation="识别模式",
            result=f"{complement_num}是合十数，{repeated_num}是重复数"
        ))
        
        # 步骤2：验证合十
        steps.append(CalculationStep(
            description=f"分析：{complement_num}={A}x10+{B}并且{A}+{B}=10",
            operation="验证合十",
            result=f"合十数是：{complement_num}"
        ))
        
        # 步骤3：应用公式 (A×10 + B) × (C×10 + C) = (A+1) × C × 100 + B × C
        first_part = (A + 1) * C * 100
        second_part = B * C
        final_result = first_part + second_part
        
        steps.append(CalculationStep(
            description=f"应用公式：(A×10+B) × (C×10+C) = (A+1) × C × 100 + B × C",
            operation="应用公式",
            result=f"= {A+1}×{C}×100 + {B}×{C} = {first_part} + {second_part}",
            formula=f"({A}×10+{B}) × ({C}×10+{C}) = ({A}+1)×{C}×100 + {B}×{C}"
        ))
        
        # 步骤4：计算最终结果
        steps.append(CalculationStep(
            description=f"计算结果：{first_part} + {second_part} = {final_result}",
            operation="计算结果",
            result=final_result
        ))
        
        return steps


"""
由高到低位法加法实现
按照数位从高到低的顺序进行加法计算，先算十位、百位等高位，再算个位等低位
这是最通用和直观的心算方法
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class HighToLowDigitAddition(MathCalculator):
    """由高到低位法加法算法"""
    
    def __init__(self):
        super().__init__("由高到低位法", "按数位从高到低顺序计算，符合心算习惯的通用加法方法", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适用于所有多位数加法（两位数及以上）"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, (int, float)) and isinstance(addend2, (int, float))):
                return False
            
            # 转换为整数
            addend1, addend2 = int(addend1), int(addend2)
            
            # 适用于两位数及以上的加法
            return addend1 >= 10 and addend2 >= 10
        except:
            return False
    
    def _get_digit_breakdown(self, number: int) -> List[tuple]:
        """将数字按位分解，返回(位名称, 位值, 数值)的列表"""
        if number == 0:
            return [("个位", 0, 0)]
        
        digits = []
        position_names = ["个位", "十位", "百位", "千位", "万位"]
        position = 0
        temp = number
        
        while temp > 0 and position < len(position_names):
            digit = temp % 10
            place_value = digit * (10 ** position)
            if digit > 0:  # 只记录非零位
                digits.append((position_names[position], digit, place_value))
            temp //= 10
            position += 1
        
        # 返回从高位到低位的顺序
        return list(reversed(digits))
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建由高到低位法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用由高到低位法",
            operation="识别算法",
            result="按位从高到低顺序计算"
        ))
        
        # 分解两个数字
        breakdown1 = self._get_digit_breakdown(addend1)
        breakdown2 = self._get_digit_breakdown(addend2)
        
        breakdown1_str = " + ".join([f"{val}" for _, _, val in breakdown1])
        breakdown2_str = " + ".join([f"{val}" for _, _, val in breakdown2])
        
        steps.append(CalculationStep(
            description=f"数字分解",
            operation="按位分解",
            result=f"{addend1} = {breakdown1_str}；{addend2} = {breakdown2_str}"
        ))
        
        # 合并同级位数，从高位到低位
        all_positions = {}
        
        # 收集所有位的值
        for pos_name, digit, place_value in breakdown1:
            if pos_name not in all_positions:
                all_positions[pos_name] = [0, 0]
            all_positions[pos_name][0] = place_value
        
        for pos_name, digit, place_value in breakdown2:
            if pos_name not in all_positions:
                all_positions[pos_name] = [0, 0]
            all_positions[pos_name][1] = place_value
        
        # 按位次顺序处理（从高到低）
        position_order = ["万位", "千位", "百位", "十位", "个位"]
        partial_results = []
        
        for pos_name in position_order:
            if pos_name in all_positions:
                val1, val2 = all_positions[pos_name]
                if val1 > 0 or val2 > 0:
                    pos_sum = val1 + val2
                    partial_results.append(pos_sum)
                    
                    if val1 > 0 and val2 > 0:
                        steps.append(CalculationStep(
                            description=f"{pos_name}相加：{val1} + {val2} = {pos_sum}",
                            operation=f"{pos_name}计算",
                            result=pos_sum
                        ))
                    elif val1 > 0:
                        steps.append(CalculationStep(
                            description=f"{pos_name}：{val1}（仅第一个数有此位）",
                            operation=f"{pos_name}计算",
                            result=val1
                        ))
                    else:
                        steps.append(CalculationStep(
                            description=f"{pos_name}：{val2}（仅第二个数有此位）",
                            operation=f"{pos_name}计算",
                            result=val2
                        ))
        
        # 合并所有位的结果
        if len(partial_results) > 1:
            final_result = sum(partial_results)
            partial_str = " + ".join([str(x) for x in partial_results])
            steps.append(CalculationStep(
                description=f"合并各位结果：{partial_str} = {final_result}",
                operation="合并结果",
                result=final_result,
                formula="由高到低位法：分位计算后合并"
            ))
        else:
            final_result = partial_results[0] if partial_results else 0
            steps.append(CalculationStep(
                description=f"最终结果：{final_result}",
                operation="确认结果",
                result=final_result,
                formula="由高到低位法：分位计算后合并"
            ))
        
        return steps


"""
异分母分数减法算子 - 通分法
"""

from typing import Dict, Any
from fractions import Fraction
from math import gcd
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionUtils


class DifferentDenominatorSubtractCalculator(FractionCalculator):
    """异分母分数减法算子"""
    
    def __init__(self):
        super().__init__("异分母减法", "通分后相减分子", priority=7)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配异分母减法问题"""
        if problem.operation != OperationType.SUBTRACT:
            return {"matched": False, "score": 0.0, "reason": "不是减法"}
        
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        
        # 检查是否为异分母
        frac1, frac2 = problem.operands[0].fraction, problem.operands[1].fraction
        different_denom = frac1.denominator != frac2.denominator
        
        if different_denom:
            return {"matched": True, "score": 1.0, "reason": "异分母减法"}
        else:
            return {"matched": False, "score": 0.0, "reason": "分母相同"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行异分母减法"""
        try:
            operand1, operand2 = problem.operands
            frac1, frac2 = operand1.fraction, operand2.fraction
            steps = []
            
            # 步骤1：计算最小公倍数(LCM)
            lcm_value = FractionUtils.lcm(frac1.denominator, frac2.denominator)
            
            steps.append(FractionStep(
                description=f"计算分母的最小公倍数：lcm({frac1.denominator}, {frac2.denominator}) = {lcm_value}",
                operation="计算分母的最小公倍数",
                result=f"lcm = {lcm_value}",
                formula=f"公分母：{lcm_value}"
            ))
            
            # 步骤2：通分（调整分子）
            multiplier1 = lcm_value // frac1.denominator
            multiplier2 = lcm_value // frac2.denominator
            
            new_num1 = frac1.numerator * multiplier1
            new_num2 = frac2.numerator * multiplier2
            
            adjusted_frac1 = Fraction(new_num1, lcm_value)
            adjusted_frac2 = Fraction(new_num2, lcm_value)
            
            steps.append(FractionStep(
                description=f"通分：{frac1} = {frac1.numerator}×{multiplier1}/{frac1.denominator}×{multiplier1} = {adjusted_frac1}；"
                           f"{frac2} = {frac2.numerator}×{multiplier2}/{frac2.denominator}×{multiplier2} = {adjusted_frac2}",
                operation="通分调整分子",
                result=f"{adjusted_frac1} - {adjusted_frac2}",
                formula="扩分子保持分数值不变"
            ))
            
            # 步骤3：分子相减
            diff_numerator = new_num1 - new_num2
            result_fraction = Fraction(diff_numerator, lcm_value)
            
            steps.append(FractionStep(
                description=f"分子相减：{new_num1} - {new_num2} = {diff_numerator}",
                operation="分子相减，分母不变",
                result=f"{diff_numerator}/{lcm_value}",
                formula=f"结果 = {diff_numerator}/{lcm_value}"
            ))
            
            # 步骤4：约分
            reduced_result = result_fraction
            if result_fraction.denominator != 1:
                common_factor = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if common_factor > 1:
                    reduced_result = Fraction(result_fraction.numerator // common_factor,
                                            result_fraction.denominator // common_factor)
                    steps.append(FractionStep(
                        description=f"约分：{result_fraction} → {reduced_result}",
                        operation="约分",
                        result=str(reduced_result),
                        formula=f"约去公因子{common_factor}"
                    ))
            
            # 步骤5：根据需要转为带分数
            final_result = reduced_result
            if (problem.display_mixed and 
                abs(reduced_result.numerator) >= reduced_result.denominator and 
                reduced_result.denominator != 1):
                
                whole = reduced_result.numerator // reduced_result.denominator
                remainder = abs(reduced_result.numerator) % reduced_result.denominator
                
                if remainder == 0:
                    final_result = str(whole)
                else:
                    if reduced_result.numerator < 0:
                        final_result = f"-{abs(whole)} {remainder}/{reduced_result.denominator}"
                    else:
                        final_result = f"{whole} {remainder}/{reduced_result.denominator}"
                
                steps.append(FractionStep(
                    description=f"转为带分数：{reduced_result} = {final_result}",
                    operation="转为带分数",
                    result=str(final_result)
                ))
            
            return FractionResult(
                success=True,
                result=final_result,
                steps=steps,
                step_count=len(steps),
                formula=f"异分母减法：{frac1} - {frac2} = {final_result}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"异分母减法计算失败: {str(e)}"
            )



"""
两位数的平方实现
利用完全平方公式(a+b)²=a²+2ab+b²计算两位数平方
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class TwoDigitSquare(MathCalculator):
    """两位数的平方"""
    
    def __init__(self):
        super().__init__("两位数平方", "利用完全平方公式计算两位数平方", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两位数的平方"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, int) and isinstance(b, int)):
                return False
            
            # 两个数必须相等且为两位数
            return (a == b and 10 <= abs(a) <= 99)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建两位数平方步骤"""
        numbers = formula.get_numbers()
        num = int(numbers[0].get_numeric_value())
        
        # 分解为十位和个位
        tens = num // 10
        ones = num % 10
        
        steps = []
        
        # 检查是否为个位数是5的特殊情况
        if ones == 5:
            return self._construct_ends_with_five_steps(num, tens)
        else:
            return self._construct_general_steps(num, tens, ones)
    
    def _construct_ends_with_five_steps(self, num: int, tens: int) -> List[CalculationStep]:
        """构建个位数是5的平方步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{num}² 使用个位是5的特殊公式",
            operation="识别特殊模式",
            result="个位是5的数平方：前面十位×(十位+1)，后面写25"
        ))
        
        steps.append(CalculationStep(
            description=f"提取十位数：{tens}",
            operation="提取十位",
            result=f"十位数 a = {tens}"
        ))
        
        # 计算十位×(十位+1)
        front_part = tens * (tens + 1)
        
        steps.append(CalculationStep(
            description=f"计算前半部分：{tens} × ({tens}+1) = {tens} × {tens + 1} = {front_part}",
            operation="计算前半部分",
            result=front_part
        ))
        
        steps.append(CalculationStep(
            description="后半部分固定为25",
            operation="确定后半部分",
            result=25
        ))
        
        # 最终结果
        final_result = front_part * 100 + 25
        
        steps.append(CalculationStep(
            description=f"组合结果：{front_part}|25 = {final_result}",
            operation="组合前后部分",
            result=final_result,
            formula="(10a+5)² = a×(a+1)×100 + 25"
        ))
        
        return steps
    
    def _construct_general_steps(self, num: int, tens: int, ones: int) -> List[CalculationStep]:
        """构建普通两位数平方步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{num}² 使用两位数平方公式",
            operation="识别模式",
            result="使用(a+b)²=a²+2ab+b²公式"
        ))
        
        steps.append(CalculationStep(
            description=f"分解：{num} = {tens * 10} + {ones}",
            operation="数字分解",
            result=f"十位：{tens}，个位：{ones}"
        ))
        
        # 计算各部分
        tens_square = (tens * 10) ** 2
        cross_product = 2 * (tens * 10) * ones
        ones_square = ones ** 2
        
        steps.append(CalculationStep(
            description=f"首平方：({tens * 10})² = {tens_square}",
            operation="计算首平方",
            result=tens_square
        ))
        
        steps.append(CalculationStep(
            description=f"首尾两倍：2 × {tens * 10} × {ones} = {cross_product}",
            operation="计算首尾两倍",
            result=cross_product
        ))
        
        steps.append(CalculationStep(
            description=f"尾平方：{ones}² = {ones_square}",
            operation="计算尾平方", 
            result=ones_square
        ))
        
        # 最终结果
        final_result = tens_square + cross_product + ones_square
        
        steps.append(CalculationStep(
            description=f"组合结果：{tens_square} + {cross_product} + {ones_square} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="(a+b)² = a² + 2ab + b²"
        ))
        
        return steps


"""
由高到低位法减法实现
从最高位开始，逐位相减，适用于无需借位的减法
符合心算习惯的通用减法方法
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class HighToLowDigitSubtraction(MathCalculator):
    """由高到低位法减法算法"""
    
    def __init__(self):
        super().__init__("由高到低位法减法", "按数位从高到低顺序计算，符合心算习惯", priority=4)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适用于不需要借位的多位数减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, (int, float)) and isinstance(subtrahend, (int, float))):
                return False
            
            # 转换为整数
            minuend, subtrahend = int(minuend), int(subtrahend)
            
            # 适用于两位数及以上的减法，且被减数大于减数
            if minuend < 10 or subtrahend < 10 or minuend <= subtrahend:
                return False
            
            # 检查是否需要借位（如果需要借位，优先级应该较低）
            return not self._needs_borrowing(minuend, subtrahend)
            
        except:
            return False
    
    def _needs_borrowing(self, minuend: int, subtrahend: int) -> bool:
        """检查是否需要借位"""
        minuend_str = str(minuend)
        subtrahend_str = str(subtrahend).zfill(len(minuend_str))
        
        for i in range(len(minuend_str)):
            if int(minuend_str[i]) < int(subtrahend_str[i]):
                return True
        return False
    
    def _get_digit_breakdown(self, number: int) -> List[tuple]:
        """将数字按位分解，返回(位名称, 位值, 数值)的列表"""
        if number == 0:
            return [("个位", 0, 0)]
        
        digits = []
        position_names = ["个位", "十位", "百位", "千位", "万位"]
        position = 0
        temp = number
        
        while temp > 0 and position < len(position_names):
            digit = temp % 10
            place_value = digit * (10 ** position)
            if digit > 0:  # 只记录非零位
                digits.append((position_names[position], digit, place_value))
            temp //= 10
            position += 1
        
        # 返回从高位到低位的顺序
        return list(reversed(digits))
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建由高到低位法步骤"""
        numbers = formula.get_numbers()
        minuend = int(numbers[0].get_numeric_value())
        subtrahend = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用由高到低位法",
            operation="识别算法",
            result="按位从高到低顺序计算"
        ))
        
        # 分解两个数字
        breakdown_minuend = self._get_digit_breakdown(minuend)
        breakdown_subtrahend = self._get_digit_breakdown(subtrahend)
        
        breakdown_minuend_str = " + ".join([f"{val}" for _, _, val in breakdown_minuend])
        breakdown_subtrahend_str = " + ".join([f"{val}" for _, _, val in breakdown_subtrahend])
        
        steps.append(CalculationStep(
            description=f"数字分解",
            operation="按位分解",
            result=f"{minuend} = {breakdown_minuend_str}；{subtrahend} = {breakdown_subtrahend_str}"
        ))
        
        # 合并同级位数，从高位到低位
        all_positions = {}
        
        # 收集被减数的所有位的值
        for pos_name, digit, place_value in breakdown_minuend:
            if pos_name not in all_positions:
                all_positions[pos_name] = [0, 0]
            all_positions[pos_name][0] = place_value
        
        # 收集减数的所有位的值
        for pos_name, digit, place_value in breakdown_subtrahend:
            if pos_name not in all_positions:
                all_positions[pos_name] = [0, 0]
            all_positions[pos_name][1] = place_value
        
        # 按位次顺序处理（从高到低）
        position_order = ["万位", "千位", "百位", "十位", "个位"]
        partial_results = []
        
        for pos_name in position_order:
            if pos_name in all_positions:
                val1, val2 = all_positions[pos_name]
                if val1 > 0 or val2 > 0:
                    pos_diff = val1 - val2
                    if pos_diff != 0:
                        partial_results.append(pos_diff)
                        
                        if val1 > 0 and val2 > 0:
                            steps.append(CalculationStep(
                                description=f"{pos_name}相减：{val1} - {val2} = {pos_diff}",
                                operation=f"{pos_name}计算",
                                result=pos_diff
                            ))
                        elif val1 > 0:
                            steps.append(CalculationStep(
                                description=f"{pos_name}：{val1}（减数此位为0）",
                                operation=f"{pos_name}计算",
                                result=val1
                            ))
        
        # 合并所有位的结果
        if len(partial_results) > 1:
            final_result = sum(partial_results)
            partial_str = " + ".join([str(x) for x in partial_results])
            steps.append(CalculationStep(
                description=f"合并各位结果：{partial_str} = {final_result}",
                operation="合并结果",
                result=final_result,
                formula="由高到低位法：分位计算后合并"
            ))
        else:
            final_result = partial_results[0] if partial_results else 0
            steps.append(CalculationStep(
                description=f"最终结果：{final_result}",
                operation="确认结果",
                result=final_result,
                formula="由高到低位法：分位计算后合并"
            ))
        
        return steps


"""
带分数与假分数互转算子
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionType


class MixedImproperConvertCalculator(FractionCalculator):
    """带分数与假分数互转算子"""
    
    def __init__(self):
        super().__init__("带分数假分数互转", "带分数与假分数相互转换", priority=4)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配互转问题"""
        if problem.operation != OperationType.CONVERT:
            return {"matched": False, "score": 0.0, "reason": "不是转换"}
        
        if len(problem.operands) != 1:
            return {"matched": False, "score": 0.0, "reason": "需要一个操作数"}
        
        operand = problem.operands[0]
        
        # 检查是否为带分数转假分数
        if operand.is_mixed and problem.target_format == "improper":
            return {"matched": True, "score": 1.0, "reason": "带分数转假分数"}
        
        # 检查是否为假分数转带分数
        if (operand.fraction_type == FractionType.IMPROPER and 
            problem.target_format == "mixed"):
            return {"matched": True, "score": 1.0, "reason": "假分数转带分数"}
        
        return {"matched": False, "score": 0.0, "reason": "不匹配的转换类型"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行互转"""
        try:
            operand = problem.operands[0]
            steps = []
            
            if operand.is_mixed:
                return self._mixed_to_improper(operand, steps)
            else:
                return self._improper_to_mixed(operand, steps, problem)
                
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"带分数假分数互转失败: {str(e)}"
            )
    
    def _mixed_to_improper(self, operand, steps: list) -> FractionResult:
        """带分数转假分数"""
        whole = operand.whole_part
        fraction = operand.fraction
        
        # 计算假分数
        improper_numerator = whole * fraction.denominator + fraction.numerator
        improper_fraction = Fraction(improper_numerator, fraction.denominator)
        
        steps.append(FractionStep(
            description=f"带分数转假分数：{whole} {fraction} → ({whole}×{fraction.denominator}+{fraction.numerator})/{fraction.denominator}",
            operation="to_improper_or_to_mixed",
            result=str(improper_fraction),
            formula=f"improper = whole*denom + numer = {whole}×{fraction.denominator}+{fraction.numerator} = {improper_numerator}"
        ))
        
        return FractionResult(
            success=True,
            result=improper_fraction,
            steps=steps,
            step_count=len(steps),
            formula=f"带分数转假分数：{operand.raw} → {improper_fraction}",
            validation=True
        )
    
    def _improper_to_mixed(self, operand, steps: list, problem: FractionProblem) -> FractionResult:
        """假分数转带分数"""
        fraction = operand.fraction
        
        # 计算带分数
        whole = fraction.numerator // fraction.denominator
        remainder = abs(fraction.numerator) % fraction.denominator
        
        if remainder == 0:
            # 结果是整数
            result = str(whole)
            steps.append(FractionStep(
                description=f"假分数转整数：{fraction} = {whole}",
                operation="to_integer",
                result=result
            ))
        else:
            # 结果是带分数
            if fraction.numerator < 0:
                result = f"-{abs(whole)} {remainder}/{fraction.denominator}"
            else:
                result = f"{whole} {remainder}/{fraction.denominator}"
            
            steps.append(FractionStep(
                description=f"假分数转带分数：{fraction} = {result}",
                operation="to_mixed",
                result=result,
                formula=f"整数部分={whole}, 余数={remainder}"
            ))
        
        return FractionResult(
            success=True,
            result=result,
            steps=steps,
            step_count=len(steps),
            formula=f"假分数转带分数：{fraction} → {result}",
            validation=True
        )



"""
分数转换算子 - 分数与小数、百分数互转
"""

from typing import Dict, Any
from fractions import Fraction
from decimal import Decimal, getcontext
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionType

# 设置高精度
getcontext().prec = 28


class FractionConvertCalculator(FractionCalculator):
    """分数转换算子"""
    
    def __init__(self):
        super().__init__("分数转换", "分数与小数、百分数互转", priority=12)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配转换问题"""
        if problem.operation != OperationType.CONVERT:
            return {"matched": False, "score": 0.0, "reason": "不是转换"}
        
        # 检查目标格式
        if problem.target_format in ["decimal", "percent", "latex_fraction"]:
            return {"matched": True, "score": 1.0, "reason": "分数转换"}
        
        return {"matched": False, "score": 0.0, "reason": "不支持的转换格式"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行分数转换"""
        try:
            operand = problem.operands[0]
            fraction = operand.fraction
            target_format = problem.target_format
            steps = []
            
            if target_format == "decimal":
                return self._convert_to_decimal(fraction, steps, problem)
            elif target_format == "percent":
                return self._convert_to_percent(fraction, steps, problem)
            elif target_format == "latex_fraction":
                return self._convert_to_latex(fraction, steps, problem)
            else:
                return FractionResult(
                    success=False,
                    error=f"不支持的转换格式: {target_format}"
                )
                
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"分数转换失败: {str(e)}"
            )
    
    def _convert_to_decimal(self, fraction: Fraction, steps: list, problem: FractionProblem) -> FractionResult:
        """转换为小数"""
        # 步骤1：约分分数
        reduced = fraction
        steps.append(FractionStep(
            description=f"约分分数：{fraction} → {reduced}",
            operation="约分分数",
            result=str(reduced)
        ))
        
        # 步骤2：检查分母的质因数
        denom = reduced.denominator
        temp_denom = denom
        
        # 检查是否只包含2和5的因子
        while temp_denom % 2 == 0:
            temp_denom //= 2
        while temp_denom % 5 == 0:
            temp_denom //= 5
        
        is_terminating = temp_denom == 1
        
        if is_terminating:
            # 有限小数
            decimal_result = float(reduced)
            steps.append(FractionStep(
                description=f"分母只含2和5的因子，为有限小数：{decimal_result}",
                operation="转为有限小数",
                result=str(decimal_result),
                formula="小数= 分子/分母"
            ))
        else:
            # 循环小数，按精度截断
            decimal_result = round(float(reduced), problem.precision)
            steps.append(FractionStep(
                description=f"分母含其他质因数，为循环小数，按精度{problem.precision}截断：{decimal_result}",
                operation="转为循环小数",
                result=str(decimal_result),
                formula="小数= 分子/分母"
            ))
        
        return FractionResult(
            success=True,
            result=decimal_result,
            steps=steps,
            step_count=len(steps),
            formula=f"分数转小数：{fraction} → {decimal_result}",
            validation=True
        )
    
    def _convert_to_percent(self, fraction: Fraction, steps: list, problem: FractionProblem) -> FractionResult:
        """转换为百分数"""
        # 使用精确的分数计算避免浮点数精度问题
        percent_fraction = fraction * 100
        percent_value = float(percent_fraction)
        
        # 检查是否为整数百分数
        if percent_value == int(percent_value):
            percent_value = int(percent_value)
        else:
            # 四舍五入到合理精度
            percent_value = round(percent_value, 6)
            if percent_value == int(percent_value):
                percent_value = int(percent_value)
        
        steps.append(FractionStep(
            description=f"分数转百分数：{fraction} × 100 = {percent_value}%",
            operation="分数转百分数",
            result=f"{percent_value}%",
            formula="百分数 = ( (分子/分母) × 100 ）%"
        ))
        
        return FractionResult(
            success=True,
            result=f"{percent_value}",
            steps=steps,
            step_count=len(steps),
            formula=f"分数转百分数：{fraction} → {percent_value}%",
            validation=True
        )
    
    def _convert_to_latex(self, fraction: Fraction, steps: list, problem: FractionProblem) -> FractionResult:
        """转换为LaTeX格式"""
        from ..base_types import FractionUtils
        
        latex_result = FractionUtils.fraction_to_latex(fraction, problem.display_mixed)
        
        steps.append(FractionStep(
            description=f"转换为LaTeX格式：{fraction} → {latex_result}",
            operation="调整分数格式",
            result=latex_result
        ))
        
        return FractionResult(
            success=True,
            result=latex_result,
            steps=steps,
            step_count=len(steps),
            formula=f"调整分数格式：{fraction} → {latex_result}",
            validation=True
        )



"""
商不变性质实现（同乘同除法）
被除数和除数同时乘以或除以同一个不为0的数，商不变
主要用于简化小数除法
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep
from fractions import Fraction


class QuotientInvariantDivision(MathCalculator):
    """商不变性质算法（同乘同除法）"""
    
    def __init__(self):
        super().__init__("商不变性质", "同乘同除，商不变", priority=3)
    
    def _find_simplification_factor(self, dividend, divisor) -> int:
        """寻找合适的简化因子"""
        # 优先处理小数转整数
        if isinstance(divisor, float):
            decimal_places = len(str(divisor).split('.')[-1])
            return 10 ** decimal_places
        
        if isinstance(dividend, float):
            decimal_places = len(str(dividend).split('.')[-1])
            return 10 ** decimal_places
        
        # 处理整数的公因数
        if isinstance(dividend, int) and isinstance(divisor, int):
            # 寻找最大公因数
            import math
            gcd = math.gcd(abs(dividend), abs(divisor))
            if gcd > 1:
                return gcd
        
        return 1
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：包含小数的除法或有公因数的除法"""
        if formula.type != "division":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            dividend, divisor = [elem.get_numeric_value() for elem in numbers]
            if divisor == 0:
                return False
            
            # 情况1：除数是小数（需要转换为整数）
            if isinstance(divisor, float) and not divisor.is_integer():
                return True
            
            # 情况2：被除数是小数（可以简化）
            if isinstance(dividend, float) and not dividend.is_integer():
                return True
            
            # 情况3：被除数和除数都是整数且有公因数（可以约分）
            if isinstance(dividend, int) and isinstance(divisor, int):
                import math
                gcd = math.gcd(abs(dividend), abs(divisor))
                if gcd > 1:
                    return True
            
            return False
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建商不变性质步骤"""
        numbers = formula.get_numbers()
        dividend = numbers[0].get_numeric_value()
        divisor = numbers[1].get_numeric_value()
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{dividend} ÷ {divisor} 使用商不变性质",
            operation="识别模式",
            result="同乘同除，商不变"
        ))
        
        # 判断处理策略
        if isinstance(divisor, float) and not divisor.is_integer():
            # 策略1：除数是小数，转换为整数
            decimal_places = len(str(divisor).split('.')[-1])
            multiplier = 10 ** decimal_places
            
            new_dividend = dividend * multiplier
            new_divisor = divisor * multiplier
            
            steps.append(CalculationStep(
                description=f"除数是小数，同乘以{multiplier}消除小数点",
                operation="消除小数",
                result=f"被除数和除数都乘以{multiplier}"
            ))
            
            steps.append(CalculationStep(
                description=f"({dividend} × {multiplier}) ÷ ({divisor} × {multiplier}) = {new_dividend} ÷ {int(new_divisor)}",
                operation="同乘转换",
                result=f"{new_dividend} ÷ {int(new_divisor)}"
            ))
            
            # 计算结果
            result = new_dividend / new_divisor
            
            steps.append(CalculationStep(
                description=f"{new_dividend} ÷ {int(new_divisor)} = {result}",
                operation="整数除法",
                result=result
            ))
            
        elif isinstance(dividend, float) and not dividend.is_integer():
            # 策略2：被除数是小数，可以优化
            decimal_places = len(str(dividend).split('.')[-1])
            multiplier = 10 ** decimal_places
            
            new_dividend = dividend * multiplier
            new_divisor = divisor * multiplier
            
            steps.append(CalculationStep(
                description=f"被除数是小数，同乘以{multiplier}简化计算",
                operation="简化小数",
                result=f"被除数和除数都乘以{multiplier}"
            ))
            
            steps.append(CalculationStep(
                description=f"({dividend} × {multiplier}) ÷ ({divisor} × {multiplier}) = {int(new_dividend)} ÷ {int(new_divisor)}",
                operation="同乘转换",
                result=f"{int(new_dividend)} ÷ {int(new_divisor)}"
            ))
            
            # 计算结果
            result = new_dividend / new_divisor
            
            steps.append(CalculationStep(
                description=f"{int(new_dividend)} ÷ {int(new_divisor)} = {result}",
                operation="整数除法",
                result=result
            ))
            
        else:
            # 策略3：整数约分
            import math
            gcd = math.gcd(abs(int(dividend)), abs(int(divisor)))
            
            new_dividend = int(dividend) // gcd
            new_divisor = int(divisor) // gcd
            
            steps.append(CalculationStep(
                description=f"找到最大公因数：{gcd}",
                operation="寻找公因数",
                result=f"{dividend} 和 {divisor} 的最大公因数是 {gcd}"
            ))
            
            steps.append(CalculationStep(
                description=f"同除以{gcd}进行约分",
                operation="约分简化",
                result=f"被除数和除数都除以{gcd}"
            ))
            
            steps.append(CalculationStep(
                description=f"({dividend} ÷ {gcd}) ÷ ({divisor} ÷ {gcd}) = {new_dividend} ÷ {new_divisor}",
                operation="约分结果",
                result=f"{new_dividend} ÷ {new_divisor}"
            ))
            
            # 计算结果
            result = new_dividend / new_divisor
            
            if new_divisor == 1:
                steps.append(CalculationStep(
                    description=f"{new_dividend} ÷ 1 = {new_dividend}",
                    operation="除以1",
                    result=new_dividend
                ))
                result = new_dividend
            else:
                steps.append(CalculationStep(
                    description=f"{new_dividend} ÷ {new_divisor} = {result}",
                    operation="简化除法",
                    result=result
                ))
        
        # 如果结果是整数，显示为整数
        if isinstance(result, float) and result.is_integer():
            result = int(result)
        
        steps.append(CalculationStep(
            description=f"最终结果：{dividend} ÷ {divisor} = {result}",
            operation="确定结果",
            result=result,
            formula="商不变性质：(a×k) ÷ (b×k) = a ÷ b"
        ))
        
        return steps


"""
比值等值算子 - 形如 a:b = c:d，求未知数
"""
from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType

class RatioEquationCalculator(FractionCalculator):
    """比值等值算子"""
    def __init__(self):
        super().__init__("比值等值", "比值等式求未知数", priority=2)

    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配比值等值问题 a:b = c:d，且有一个未知数。无EQUATION类型时采用CONVERT。"""
        if problem.operation not in [OperationType.CONVERT]:
            return {"matched": False, "score": 0.0, "reason": "不是等式"}
        if problem.id != "ratio_equation":
            return {"matched": False, "score": 0.0, "reason": "不是比值等式"}
        if len(problem.operands) != 4:
            return {"matched": False, "score": 0.0, "reason": "需要4个操作数"}
        # 检查有且仅有一个未知数
        unknown_count = sum(1 for op in problem.operands if getattr(op, 'is_unknown', False))
        if unknown_count != 1:
            return {"matched": False, "score": 0.0, "reason": "未知数数量不为1"}
        return {"matched": True, "score": 1.0, "reason": "比值等值方程"}

    def solve(self, problem: FractionProblem) -> FractionResult:
        """解比值等值方程 a:b = c:d，求未知数"""
        try:
            ops = problem.operands
            # 标记未知数位置
            idx_unknown = [i for i, op in enumerate(ops) if getattr(op, 'is_unknown', False)][0]
            knowns = [op.fraction.numerator for i, op in enumerate(ops) if i != idx_unknown]
            steps = []
            # a:b = c:d => a*d = b*c
            # 求未知数
            if idx_unknown == 0:
                # x:b = c:d => x = b*c/d
                b, c, d = knowns
                x = b * c // d
                steps.append(FractionStep(
                    description=f"x:b = c:d，x = b×c/d = {b}×{c}/{d} = {x}",
                    operation="未知数求解",
                    result=str(x)
                ))
            elif idx_unknown == 1:
                # a:x = c:d => x = a*d/c
                a, c, d = knowns
                x = a * d // c
                steps.append(FractionStep(
                    description=f"a:x = c:d，x = a×d/c = {a}×{d}/{c} = {x}",
                    operation="未知数求解",
                    result=str(x)
                ))
            elif idx_unknown == 2:
                # a:b = x:d => x = a*d/b
                a, b, d = knowns
                x = a * d // b
                steps.append(FractionStep(
                    description=f"a:b = x:d，x = a×d/b = {a}×{d}/{b} = {x}",
                    operation="未知数求解",
                    result=str(x)
                ))
            elif idx_unknown == 3:
                # a:b = c:x => x = b*c/a
                a, b, c = knowns
                x = b * c // a
                steps.append(FractionStep(
                    description=f"a:b = c:x，x = b×c/a = {b}×{c}/{a} = {x}",
                    operation="未知数求解",
                    result=str(x)
                ))
            else:
                return FractionResult(success=False, error="未知数位置错误")
            return FractionResult(
                success=True,
                result=str(x),
                steps=steps,
                step_count=len(steps),
                formula="比值等式求解",
                validation=True
            )
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"比值等值方程求解失败: {str(e)}"
            )



"""
分段求和法加法实现
将数字按照数位或意义进行分段，分别计算各段的和，最后合并
适用于数位差异明显或有特殊结构的加法，如10+100
"""

from typing import List, Tuple
from ..base_types import MathCalculator, Formula, CalculationStep


class SegmentSumAddition(MathCalculator):
    """分段求和法加法算法"""
    
    def __init__(self):
        super().__init__("分段求和法", "按数位或意义分段计算，适用于结构清晰的加法", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：适用于数位结构差异明显或有特殊规律的加法"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            addend1, addend2 = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(addend1, (int, float)) and isinstance(addend2, (int, float))):
                return False
            
            # 转换为整数
            addend1, addend2 = int(addend1), int(addend2)
            
            # 只处理正数
            if addend1 <= 0 or addend2 <= 0:
                return False
            
            # 适用条件：
            # 1. 其中一个数是整十、整百、整千数
            # 2. 两数位数差异较大（如10+100, 25+200）
            # 3. 两数都是特殊结构（如都是整十数的倍数）
            
            def is_round_number(n):
                """判断是否为整十、整百、整千等圆整数"""
                if n < 10:
                    return False
                return n % 10 == 0 or n % 100 == 0 or n % 1000 == 0
            
            def get_digit_count(n):
                """获取数位数"""
                return len(str(n))
            
            # 条件1：至少一个是圆整数
            if is_round_number(addend1) or is_round_number(addend2):
                return True
            
            # 条件2：位数差异大于1
            if abs(get_digit_count(addend1) - get_digit_count(addend2)) > 1:
                return True
            
            # 条件3：都是特殊结构（如25+75这样的互补数，或都是5的倍数等）
            if (addend1 % 5 == 0 and addend2 % 5 == 0) or (addend1 % 25 == 0 and addend2 % 25 == 0):
                return True
            
            return False
        except:
            return False
    
    def _analyze_segments(self, number: int) -> List[Tuple[str, int]]:
        """分析数字的分段结构，返回(段名称, 段值)的列表"""
        if number == 0:
            return [("零", 0)]
        
        segments = []
        
        # 按权重分段
        if number >= 1000:
            thousands = (number // 1000) * 1000
            segments.append(("千位段", thousands))
            number -= thousands
        
        if number >= 100:
            hundreds = (number // 100) * 100
            segments.append(("百位段", hundreds))
            number -= hundreds
        
        if number >= 10:
            tens = (number // 10) * 10
            segments.append(("十位段", tens))
            number -= tens
        
        if number > 0:
            segments.append(("个位段", number))
        
        return segments
    
    def _find_meaningful_segments(self, addend1: int, addend2: int) -> Tuple[List[Tuple[str, int]], List[Tuple[str, int]]]:
        """寻找有意义的分段方式"""
        # 基本的数位分段
        segments1 = self._analyze_segments(addend1)
        segments2 = self._analyze_segments(addend2)
        
        # 特殊优化：如果一个数是另一个数的整倍数关系
        if addend2 != 0 and addend1 % addend2 == 0:
            segments1 = [(f"{addend1//addend2}倍{addend2}", addend1)]
            segments2 = [("基数", addend2)]
        elif addend1 != 0 and addend2 % addend1 == 0:
            segments1 = [("基数", addend1)]
            segments2 = [(f"{addend2//addend1}倍{addend1}", addend2)]
        
        return segments1, segments2
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建分段求和法步骤"""
        numbers = formula.get_numbers()
        addend1 = int(numbers[0].get_numeric_value())
        addend2 = int(numbers[1].get_numeric_value())
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{addend1} + {addend2} 使用分段求和法",
            operation="识别分段求和法",
            result="按数位或意义进行分段计算"
        ))
        
        # 分析分段
        segments1, segments2 = self._find_meaningful_segments(addend1, addend2)
        
        # 显示分段识别
        segments1_str = "、".join([f"{name}({value})" for name, value in segments1])
        segments2_str = "、".join([f"{name}({value})" for name, value in segments2])
        
        steps.append(CalculationStep(
            description=f"识别分段结构",
            operation="分段识别",
            result=f"{addend1} 包含: {segments1_str}；{addend2} 包含: {segments2_str}"
        ))
        
        # 列出各段数值
        all_segments = []
        all_segments.extend([(name, value) for name, value in segments1])
        all_segments.extend([(name, value) for name, value in segments2])
        
        segment_list = []
        for name, value in all_segments:
            segment_list.append(f"{name}: {value}")
        
        steps.append(CalculationStep(
            description=f"列出各段数值",
            operation="段值列举",
            result="; ".join(segment_list)
        ))
        
        # 将各段数值相加
        segment_values = [value for _, value in all_segments]
        if len(segment_values) > 1:
            sum_expression = " + ".join([str(value) for value in segment_values])
            final_result = sum(segment_values)
            
            steps.append(CalculationStep(
                description=f"将各段数值相加：{sum_expression} = {final_result}",
                operation="分段求和",
                result=final_result
            ))
        else:
            final_result = segment_values[0] if segment_values else 0
        
        steps.append(CalculationStep(
            description=f"最终答案：{final_result}",
            operation="确认结果",
            result=final_result,
            formula="分段求和法：按结构分段后逐段相加"
        ))
        
        return steps


from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep, ElementType

class RevertCalculator(MathCalculator):
	def __init__(self):
		super().__init__("逆运算算子", "加减乘除逆运算求未知数", priority=2)

	def is_match_pattern(self, formula: Formula) -> bool:
		expr = formula.original_expression.replace(" ", "")  # 移除空格后再判断
		if "=" not in expr or "@" not in expr:
			return False
		eq_index = expr.index("=")
		at_index = expr.index("@")
		if at_index > eq_index:
			return False
		operators = [el.value for el in formula.elements if el.type == ElementType.OPERATOR]
		## 暂时只支持一个运算符的逆运算
		if len(operators) != 1:
			return False
		return True

	def construct_steps(self, formula: Formula) -> List[CalculationStep]:
		expr = formula.original_expression.replace(" ","")
		eq_index = expr.index("=")
		left = expr[:eq_index]
		right = expr[eq_index+1:]
		ops = ["+", "-", "×", "*", "÷", "/"]
		op = None
		# 优先匹配非*的运算符，避免与×冲突
		for o in ops:
			if o in left and (o != "*" or "×" not in left):
				op = o
				break
		if op is None:
			return [CalculationStep(description="未识别运算符", operation="错误", result="错误")]
		# 处理未知数在左侧开头的情况（@ op 数字）
		if left.startswith("@"):
			# 正确提取数字部分（支持多字符运算符后的数字）
			num_str = left.split(op, 1)[1] if op in left else left[1:]
			num_str = num_str.strip('+-*×÷')
			try:
				num = float(num_str)
				right_val = float(right)
			except:
				return [CalculationStep(description="数字解析失败", operation="错误", result="错误")]
			if op == "+":
				x = right_val - num
				steps = [
					CalculationStep(description=f"设未知数为x，x+{num}={right_val}", operation="逆加法", result=f"x={right_val}-{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "-":
				x = right_val + num
				steps = [
					CalculationStep(description=f"设未知数为x，x-{num}={right_val}", operation="逆减法", result=f"x={right_val}+{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "*" or op == "×":
				x = right_val / num
				steps = [
					CalculationStep(description=f"设未知数为x，x×{num}={right_val}", operation="逆乘法", result=f"x={right_val}÷{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "÷":
				x = right_val * num
				steps = [
					CalculationStep(description=f"设未知数为x，x÷{num}={right_val}", operation="逆除法", result=f"x={right_val}×{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			else:
				steps = [CalculationStep(description="不支持的运算符", operation="错误", result="错误")]
			return steps
		else:
			# 处理数字 op 未知数的情况
			parts = left.split(op, 1)  # 只分割一次，避免数字中包含运算符
			if len(parts) != 2:
				return [CalculationStep(description="表达式格式错误", operation="错误", result="错误")]
			num_str, unknown_str = parts[0], parts[1]
			num_str = num_str.strip('+-*×÷')
			# 验证未知数格式
			if unknown_str.strip() != "@":
				return [CalculationStep(description="表达式格式错误", operation="错误", result="错误")]
			try:
				num = float(num_str)
				right_val = float(right)
			except:
				return [CalculationStep(description="数字解析失败", operation="错误", result="错误")]
			if op == "+":
				x = right_val - num
				steps = [
					CalculationStep(description=f"设未知数为x，{num}+x={right_val}", operation="逆加法", result=f"x={right_val}-{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "-":
				x = num - right_val
				steps = [
					CalculationStep(description=f"设未知数为x，{num}-x={right_val}", operation="逆减法", result=f"x={num}-{right_val}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "*"  or op == "×":
				x = right_val / num
				steps = [
					CalculationStep(description=f"设未知数为x，{num}×x={right_val}", operation="逆乘法", result=f"x={right_val}÷{num}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			elif op == "÷":
				x = num / right_val
				steps = [
					CalculationStep(description=f"设未知数为x，{num}÷x={right_val}", operation="逆除法", result=f"x={num}÷{right_val}"),
					CalculationStep(description="计算x", operation="计算", result=x)
				]
			else:
				steps = [CalculationStep(description="不支持的运算符", operation="错误", result="错误")]
			return steps


"""
混合运算表达式解析器
"""

import re
from typing import List, Tuple, Optional
from decimal import Decimal
from .base_types import MixedProblem, MixedOperand, NumberType, OperationType


class MixedParser:
    """混合运算解析器"""
    
    # 操作符优先级
    OPERATOR_PRECEDENCE = {
        '+': 1,
        '-': 1,
        '*': 2,
        '×': 2,
        '/': 2,
        '÷': 2,
        '(': 0,
        ')': 0
    }
    
    @classmethod
    def parse(cls, expression: str) -> MixedProblem:
        """
        解析混合运算表达式
        
        Args:
            expression: 表达式字符串
            
        Returns:
            MixedProblem: 解析后的问题对象
            
        Raises:
            ValueError: 当表达式格式错误时
        """
        # 输入验证
        if not expression or not expression.strip():
            raise ValueError("表达式不能为空")
        
        if '=' not in expression and '@' in expression:
            expression = re.sub(r'@([\d\-]+)', '=@', expression)
        eq_index = expression.find('=')
        answer = ""
        if eq_index != -1:
            # 处理等式，暂时只解析等号左侧
            expression = expression[:eq_index]
            answer_part = expression[eq_index + 1:]
            if answer_part and answer_part != '@' and answer_part != '?':
                answer = answer_part
        # 清理表达式
        expression = expression.replace(' ', '').replace('x', '*').replace('X', '*')
        # 标准化表达式
        normalized_expr = cls._normalize_expression(expression.strip())
        
        # 验证表达式格式
        cls._validate_expression(normalized_expr)
        
        # 检查括号
        has_parentheses = '(' in normalized_expr or ')' in normalized_expr
        
        # 提取操作数和操作符
        operands, operations = cls._extract_operands_and_operations(normalized_expr)
        
        # 验证解析结果
        if not operands:
            raise ValueError("表达式中没有找到有效的数字")
        
        # 计算复杂度等级
        complexity_level = cls._calculate_complexity(operands, operations, has_parentheses)
        
        # 查找特殊数字
        special_numbers = cls._find_special_numbers(operands)
        
        return MixedProblem(
            expression=normalized_expr,
            operands=operands,
            operations=operations,
            has_parentheses=has_parentheses,
            complexity_level=complexity_level,
            special_numbers=special_numbers
        )
    
    @classmethod
    def _normalize_expression(cls, expression: str) -> str:
        """标准化表达式"""
        # 移除空格
        expr = expression.replace(' ', '')
        
        # 统一运算符
        expr = expr.replace('×', '*').replace('÷', '/')
        
        # 处理负号 - 只在开头或括号后处理
        expr = re.sub(r'^-', '0-', expr)  # 开头的负号
        expr = re.sub(r'\(-', '(0-', expr)  # 括号后的负号
        
        return expr
    
    @classmethod
    def _validate_expression(cls, expression: str):
        """验证表达式格式"""
        # 检查是否包含无效字符
        valid_chars = set('0123456789+-*/.()×÷')
        invalid_chars = set(expression) - valid_chars
        if invalid_chars:
            raise ValueError(f"表达式包含无效字符: {', '.join(invalid_chars)}")
        
        # 检查括号匹配
        if not cls._check_parentheses_balance(expression):
            raise ValueError("括号不匹配")
        
        # 检查运算符连续
        if re.search(r'[+\-*/]{2,}', expression):
            raise ValueError("运算符不能连续出现")
        
        # 检查表达式开头和结尾
        if re.match(r'^[+*/]', expression):
            raise ValueError("表达式不能以 +, *, / 开头")
        
        if re.search(r'[+\-*/]$', expression):
            raise ValueError("表达式不能以运算符结尾")
    
    @classmethod
    def _check_parentheses_balance(cls, expression: str) -> bool:
        """检查括号是否匹配"""
        count = 0
        for char in expression:
            if char == '(':
                count += 1
            elif char == ')':
                count -= 1
                if count < 0:  # 右括号多于左括号
                    return False
        return count == 0  # 最终应该平衡
    
    @classmethod
    def _extract_operands_and_operations(cls, expression: str) -> Tuple[List[MixedOperand], List[str]]:
        """提取操作数和操作符 - 简化版本，确保正确解析"""
        operands = []
        operations = []
        
        # 使用更简单的方法：逐字符解析
        i = 0
        current_number = ""
        position = 0
        
        while i < len(expression):
            char = expression[i]
            
            if char.isdigit() or char == '.':
                current_number += char
            elif char in '+-*/()':
                # 保存当前数字
                if current_number:
                    try:
                        if '.' in current_number:
                            value = float(current_number)
                            number_type = NumberType.DECIMAL
                        else:
                            value = int(current_number)
                            number_type = NumberType.INTEGER
                        
                        operand = MixedOperand(
                            value=value,
                            original=current_number,
                            number_type=number_type,
                            position=position
                        )
                        operands.append(operand)
                        position += 1
                        current_number = ""
                    except ValueError:
                        raise ValueError(f"无法解析数字: {current_number}")
                
                # 保存运算符
                operations.append(char)
            
            i += 1
        
        # 保存最后一个数字
        if current_number:
            try:
                if '.' in current_number:
                    value = float(current_number)
                    number_type = NumberType.DECIMAL
                else:
                    value = int(current_number)
                    number_type = NumberType.INTEGER
                
                operand = MixedOperand(
                    value=value,
                    original=current_number,
                    number_type=number_type,
                    position=position
                )
                operands.append(operand)
            except ValueError:
                raise ValueError(f"无法解析数字: {current_number}")
        
        return operands, operations
    
    @classmethod
    def _is_number(cls, token: str) -> bool:
        """检查token是否为有效数字"""
        try:
            float(token)
            return True
        except ValueError:
            return False
    
    @classmethod
    def _calculate_complexity(cls, operands: List[MixedOperand], operations: List[str], has_parentheses: bool) -> int:
        """计算复杂度等级"""
        complexity = 1
        
        # 操作数数量影响复杂度
        if len(operands) > 3:
            complexity += 1
        if len(operands) > 5:
            complexity += 1
        
        # 运算类型影响复杂度
        has_multiply_divide = any(op in ['*', '/'] for op in operations if op not in ['(', ')'])
        has_add_subtract = any(op in ['+', '-'] for op in operations if op not in ['(', ')'])
        
        if has_multiply_divide and has_add_subtract:
            complexity += 1  # 混合运算
        
        # 括号增加复杂度
        if has_parentheses:
            complexity += 1
        
        # 小数增加复杂度
        if any(op.number_type == NumberType.DECIMAL for op in operands):
            complexity += 1
        
        return min(complexity, 5)  # 最大复杂度为5
    
    @classmethod
    def _find_special_numbers(cls, operands: List[MixedOperand]) -> List[int]:
        """查找特殊数字的位置"""
        special_positions = []
        for i, operand in enumerate(operands):
            if operand.value in [0, 1, -1, 10, 100, 0.1, 0.5]:
                special_positions.append(i)
        return special_positions



"""
质量单位换算算子
"""

from typing import List, Dict
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep


class MassUnitCalculator(UnitCalculator):
    """质量单位换算算子"""
    
    def __init__(self):
        super().__init__("质量单位换算", "处理吨、千克、克之间的换算", UnitType.MASS)
    
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        """初始化质量单位换算规则"""
        return {
            # 吨换算
            "t": {"t": 1, "kg": 1000, "g": 1000000},
            "吨": {"吨": 1, "千克": 1000, "公斤": 1000, "克": 1000000},
            
            # 千克换算
            "kg": {"t": 0.001, "kg": 1, "g": 1000},
            "千克": {"吨": 0.001, "千克": 1, "公斤": 1, "克": 1000},
            "公斤": {"吨": 0.001, "千克": 1, "公斤": 1, "克": 1000},
            
            # 克换算
            "g": {"t": 0.000001, "kg": 0.001, "g": 1},
            "克": {"吨": 0.000001, "千克": 0.001, "公斤": 0.001, "克": 1},
        }
    
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        """判断是否为质量单位换算问题"""
        return problem.unit_type == UnitType.MASS

    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        """求解质量单位换算问题，支持 operator（+/-）单位运算"""
        steps = []
        total_result = 0
        operator = "+"
        for source_value in problem.source_values:
            conversion_factor = self.get_conversion_factor(source_value.unit, problem.target_unit)
            if conversion_factor is None:
                raise ValueError(f"无法从{source_value.unit}换算到{problem.target_unit}")
            converted_value = source_value.value * conversion_factor
            if operator == '-':
                total_result -= converted_value
            elif operator == '+':
                total_result += converted_value
            operator = getattr(source_value, 'operator', '+')
            # 生成标准化的换算公式
            formula = self._get_standard_formula(source_value.unit, problem.target_unit)
            step = ConversionStep(
                description=f"换算{source_value.unit}到{problem.target_unit}",
                operation="质量单位换算",
                from_value=source_value,
                result=UnitValue(converted_value, problem.target_unit),
                formula=formula
            )
            steps.append(step)
        if len(problem.source_values) > 1:
            formula_parts = []
            for v in problem.source_values:
                factor = self.get_conversion_factor(v.unit, problem.target_unit)
                if factor is not None:
                    formula_parts.append(str(v.value * factor) + getattr(v, 'operator', '+'))
                else:
                    formula_parts.append(str(v.value) + getattr(v, 'operator', '+'))
            formula = ' '.join(formula_parts).rstrip('+')
            step = ConversionStep(
                description=f"合并质量换算结果",
                operation="质量求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(total_result, problem.target_unit),
                formula=f"总质量 = {formula}"
            )
            steps.append(step)
        return total_result, steps
    
    def _get_standard_formula(self, from_unit: str, to_unit: str) -> str:
        """获取标准化的换算公式（大单位 = 小单位倍数）"""
        # 定义单位大小顺序（从大到小）
        unit_hierarchy = {
            "t": 3, "吨": 3,
            "kg": 2, "千克": 2, "公斤": 2,
            "g": 1, "克": 1
        }
        
        from_level = unit_hierarchy.get(from_unit, 0)
        to_level = unit_hierarchy.get(to_unit, 0)
        
        # 确定哪个是大单位，哪个是小单位
        if from_level > to_level:
            big_unit, small_unit = from_unit, to_unit
            factor = self.get_conversion_factor(from_unit, to_unit)
        else:
            big_unit, small_unit = to_unit, from_unit
            conv = self.get_conversion_factor(from_unit, to_unit)
            factor = 1 / conv if conv and conv != 0 else 1
        if factor and factor != 1:
            if factor == int(factor):
                return f"1{big_unit} = {int(factor)}{small_unit}"
            else:
                return f"1{big_unit} = {factor}{small_unit}"
        else:
            return f"1{from_unit} = 1{to_unit}"



"""
混合/连续运算计算引擎
"""

from typing import Dict, Any, List, Optional
from .base_types import MixedProblem, MixedResult, MixedCalculator
from .parser import MixedParser
from .registry import MixedCalculatorRegistry
from ..base.engine import BaseEngine



class MixedEngine(BaseEngine):
    """混合运算计算引擎"""
    
    def __init__(self):
        self.registry = MixedCalculatorRegistry()
    
    def is_match_pattern(self, expression: str) -> bool:
        """判断是否为混合/连续运算问题"""
        cleaned_expr = self.clean_expression(expression)
        operators = self.get_operators(cleaned_expr)
        if len(operators) < 2:
            return False  # 至少需要2个运算符才算多次运算
        try:
            problem = MixedParser.parse(cleaned_expr)
            matching = self.registry.find_matching_calculators(problem)
            return len(matching) > 0
        except (ValueError, Exception):
            return False

    def solve(self, expression: str) -> Dict[str, Any]:
        """解决混合运算问题"""
        try:
            # 清理表达式，移除等号和@
            cleaned_expr = expression.replace('=@', '').replace('=', '').replace('@', '').strip()
            if not cleaned_expr:
                return {
                    'success': False,
                    'expression': expression,
                    'error': '表达式为空',
                    'name': None,
                    'result': None
                }
            
            # 统一运算符
            cleaned_expr = cleaned_expr.replace('x', '*').replace('×', '*').replace('÷', '/')
            
            # 解析表达式
            problem = MixedParser.parse(cleaned_expr)
            
            # 找到最佳算子
            calculator = self.registry.get_best_calculator(problem)
            
            if calculator is None:
                return {
                    'success': False,
                    'expression': expression,
                    'error': '未找到匹配的算子',
                    'name': None,
                    'result': None
                }
            
            # 执行计算
            result = calculator.solve(problem)
            
            if result.success:
                return {
                    'success': True,
                    'expression': expression,
                    'name': calculator.name,
                    'problem': expression,
                    'description': calculator.description,
                    'result': self.format_result(result.result),
                    'steps': result.steps,
                    'step_count': result.step_count,
                    'formula': result.formula,
                    'validation': result.validation,
                    'priority': calculator.priority,
                    'technique_used': result.technique_used
                }
            else:
                return {
                    'success': False,
                    'expression': expression,
                    'name': calculator.name,
                    'error': result.error,
                    'result': None
                }
                
        except Exception as e:
            return {
                'success': False,
                'expression': expression,
                'error': f"计算失败: {str(e)}",
                'name': None,
                'result': None
            }
    
    def solve_all_solutions(self, expression: str) -> Dict[str, Any]:
        """获取所有可能的解决方案"""
        try:
            problem = MixedParser.parse(expression)
            matching = self.registry.find_matching_calculators(problem)
            
            if not matching:
                return {
                    'success': False,
                    'expression': expression,
                    'error': '未找到匹配的算子',
                    'solutions': []
                }
            
            solutions = []
            for calculator in matching:
                try:
                    result = calculator.solve(problem)
                    if result.success:
                        solutions.append({
                            'name': calculator.name,
                            'description': calculator.description,
                            'result': result.result,
                            'steps': [self._format_step(step) for step in result.steps],
                            'step_count': result.step_count,
                            'formula': result.formula,
                            'validation': result.validation,
                            'priority': calculator.priority,
                            'technique_used': result.technique_used
                        })
                except Exception as e:
                    solutions.append({
                        'name': calculator.name,
                        'error': str(e),
                        'success': False
                    })
            
            return {
                'success': True,
                'expression': expression,
                'solutions': solutions,
                'solution_count': len(solutions)
            }
            
        except Exception as e:
            return {
                'success': False,
                'expression': expression,
                'error': f"批量计算失败: {str(e)}",
                'solutions': []
            }
    
    def _format_step(self, step) -> Dict[str, Any]:
        """格式化计算步骤"""
        return {
            'description': step.description,
            'operation': step.operation,
            'inputs': step.inputs,
            'result': step.result,
            'formula': step.formula,
            'validation': step.validation,
            'meta': step.meta
        }
    
    def get_available_methods(self) -> List[Dict[str, Any]]:
        """获取所有可用的算子"""
        methods = []
        for calc in self.registry.get_all_calculators():
            methods.append({
                'name': calc.name,
                'description': calc.description,
                'priority': calc.priority
            })
        return methods
    
    def batch_solve(self, expressions: List[str]) -> List[Dict[str, Any]]:
        """批量解决问题"""
        results = []
        for expr in expressions:
            result = self.solve(expr)
            results.append(result)
        return results



"""
面积单位换算算子
支持所有平方相关单位（中英文）
"""
from typing import List, Dict
from ..base_types import UnitCalculator, UnitProblem, UnitType, UnitValue, ConversionStep

class AreaUnitCalculator(UnitCalculator):
    """面积单位换算算子"""
    def __init__(self):
        super().__init__("面积单位换算", "处理平方米、平方分米、平方厘米、平方毫米等单位换算", UnitType.AREA)
    def _initialize_conversion_rules(self) -> Dict[str, Dict[str, float]]:
        return {
            # 平方千米
            "km²": {"km²": 1, "平方千米": 1, "公顷": 100, "m²": 1000000, "平方米": 1000000},
            "平方千米": {"km²": 1, "平方千米": 1, "公顷": 100, "m²": 1000000, "平方米": 1000000},
            # 公顷
            "ha": {"km²": 0.01, "平方千米": 0.01, "公顷": 1, "m²": 10000, "平方米": 10000},
            "公顷": {"km²": 0.01, "平方千米": 0.01, "公顷": 1, "m²": 10000, "平方米": 10000},
            # 平方米
            "m²": {"km²": 0.000001, "平方千米": 0.000001, "ha": 0.0001, "公顷": 0.0001, "m²": 1, "平方米": 1, "dm²": 100, "平方分米": 100, "cm²": 10000, "平方厘米": 10000, "mm²": 1000000, "平方毫米": 1000000},
            "平方米": {"km²": 0.000001, "平方千米": 0.000001, "ha": 0.0001, "公顷": 0.0001, "m²": 1, "平方米": 1, "dm²": 100, "平方分米": 100, "cm²": 10000, "平方厘米": 10000, "mm²": 1000000, "平方毫米": 1000000},
            # 平方分米
            "dm²": {"km²": 0.00000001, "平方千米": 0.00000001, "ha": 0.00001, "公顷": 0.00001, "m²": 0.01, "平方米": 0.01, "dm²": 1, "平方分米": 1, "cm²": 100, "平方厘米": 100, "mm²": 10000, "平方毫米": 10000},
            "平方分米": {"km²": 0.00000001, "平方千米": 0.00000001, "ha": 0.00001, "公顷": 0.00001, "m²": 0.01, "平方米": 0.01, "dm²": 1, "平方分米": 1, "cm²": 100, "平方厘米": 100, "mm²": 10000, "平方毫米": 10000},
            # 平方厘米
            "cm²": {"km²": 0.0000000001, "平方千米": 0.0000000001, "ha": 0.000001, "公顷": 0.000001, "m²": 0.0001, "平方米": 0.0001, "dm²": 0.01, "平方分米": 0.01, "cm²": 1, "平方厘米": 1, "mm²": 100, "平方毫米": 100},
            "平方厘米": {"km²": 0.0000000001, "平方千米": 0.0000000001, "ha": 0.000001, "公顷": 0.000001, "m²": 0.0001, "平方米": 0.0001, "dm²": 0.01, "平方分米": 0.01, "cm²": 1, "平方厘米": 1, "mm²": 100, "平方毫米": 100},
            # 平方毫米
            "mm²": {"km²": 0.000000000001, "平方千米": 0.000000000001, "ha": 0.0000001, "公顷": 0.0000001, "m²": 0.000001, "平方米": 0.000001, "dm²": 0.0001, "平方分米": 0.0001, "cm²": 0.01, "平方厘米": 0.01, "mm²": 1, "平方毫米": 1},
            "平方毫米": {"km²": 0.000000000001, "平方千米": 0.000000000001, "ha": 0.0000001, "公顷": 0.0000001, "m²": 0.000001, "平方米": 0.000001, "dm²": 0.0001, "平方分米": 0.0001, "cm²": 0.01, "平方厘米": 0.01, "mm²": 1, "平方毫米": 1},
        }
    def is_match_pattern(self, problem: UnitProblem) -> bool:
        return problem.unit_type == UnitType.AREA
    def solve(self, problem: UnitProblem) -> tuple[float, List[ConversionStep]]:
        steps = []
        total_result = 0
        operator = "+"
        for source_value in problem.source_values:
            conversion_factor = self.get_conversion_factor(source_value.unit, problem.target_unit)
            if conversion_factor is None:
                raise ValueError(f"无法从{source_value.unit}换算到{problem.target_unit}")
            converted_value = source_value.value * conversion_factor
            if operator == '-':
                total_result -= converted_value
            elif operator == '+':
                total_result += converted_value
            operator = getattr(source_value, 'operator', '+')
            formula = self._get_standard_formula(source_value.unit, problem.target_unit)
            step = ConversionStep(
                description=f"换算{source_value.unit}到{problem.target_unit}",
                operation="面积单位换算",
                from_value=source_value,
                result=UnitValue(converted_value, problem.target_unit),
                formula=formula
            )
            steps.append(step)
        if len(problem.source_values) > 1:
            formula_parts = []
            for v in problem.source_values:
                factor = self.get_conversion_factor(v.unit, problem.target_unit)
                if factor is not None:
                    formula_parts.append(str(v.value * factor) + getattr(v, 'operator', '+'))
                else:
                    formula_parts.append(str(v.value) + getattr(v, 'operator', '+'))
            formula = ' '.join(formula_parts).rstrip('+')
            step = ConversionStep(
                description=f"合并面积换算结果",
                operation="面积求和",
                from_value=UnitValue(0, problem.target_unit),
                result=UnitValue(total_result, problem.target_unit),
                formula=f"总面积 = {formula}"
            )
            steps.append(step)
        return total_result, steps
    def _get_standard_formula(self, from_unit: str, to_unit: str) -> str:
        unit_hierarchy = {
            "km²": 6, "平方千米": 6,
            "ha": 5, "公顷": 5,
            "m²": 4, "平方米": 4,
            "dm²": 3, "平方分米": 3,
            "cm²": 2, "平方厘米": 2,
            "mm²": 1, "平方毫米": 1
        }
        from_level = unit_hierarchy.get(from_unit, 0)
        to_level = unit_hierarchy.get(to_unit, 0)
        if from_level > to_level:
            big_unit, small_unit = from_unit, to_unit
            factor = self.get_conversion_factor(from_unit, to_unit)
        else:
            big_unit, small_unit = to_unit, from_unit
            conv = self.get_conversion_factor(from_unit, to_unit)
            factor = 1 / conv if conv and conv != 0 else 1
        if factor and factor != 1:
            if factor == int(factor):
                return f"1{big_unit} = {int(factor)}{small_unit}"
            else:
                return f"1{big_unit} = {factor}{small_unit}"
        else:
            return f"1{from_unit} = 1{to_unit}"



"""
乘法交换结合法实现
利用乘法交换律和结合律，将容易计算的数先相乘
适用于多个数相乘，其中存在互补数（如25和4）的情况
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class CommutativeAssociativeMultiplication(MathCalculator):
    """乘法交换结合法算法"""
    
    def __init__(self):
        super().__init__("交换结合法乘法", "重新排列乘法因子，先计算便于运算的组合", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：包含互补因子的乘法表达式"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) < 3:  # 至少需要3个数才有重排的意义
            return False
        
        try:
            values = [elem.get_numeric_value() for elem in numbers]
            if not all(isinstance(v, (int, float)) for v in values):
                return False
            
            # 转换为整数列表
            int_values = [int(v) for v in values]
            
            # 检查是否有有用的组合
            return self._find_useful_combinations(int_values) is not None
            
        except:
            return False
    
    def _find_useful_combinations(self, values: List[int]):
        """寻找有用的因子组合"""
        n = len(values)
        
        # 定义一些有用的组合规则
        useful_pairs = [
            (2, 5, 10), (4, 25, 100), (8, 125, 1000),
            (2, 50, 100), (4, 250, 1000), (5, 20, 100),
            (5, 2, 10), (25, 4, 100), (125, 8, 1000)
        ]
        
        # 检查两两组合
        for i in range(n):
            for j in range(i + 1, n):
                val1, val2 = values[i], values[j]
                product = val1 * val2
                
                # 检查是否是整十、整百、整千等
                if product in [10, 100, 1000] or product % 10 == 0:
                    remaining = [values[k] for k in range(n) if k != i and k != j]
                    return (i, j, val1, val2, product, remaining)
        
        # 检查三个数的组合
        for i in range(n):
            for j in range(i + 1, n):
                for k in range(j + 1, n):
                    val1, val2, val3 = values[i], values[j], values[k]
                    
                    # 检查是否有两两组合后与第三个数相乘得到整数
                    pairs = [(val1, val2, val3), (val1, val3, val2), (val2, val3, val1)]
                    for pair in pairs:
                        a, b, c = pair
                        if (a * b) % 10 == 0 or (a * b) in [10, 100, 1000]:
                            remaining = [values[m] for m in range(n) if m not in [i, j, k]]
                            return (i, j, k, a, b, c, a * b, remaining)
        
        return None
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建交换结合法步骤"""
        numbers = formula.get_numbers()
        values = [int(elem.get_numeric_value()) for elem in numbers]
        
        # 找到有用的组合
        combination = self._find_useful_combinations(values)
        if combination is None:
            raise ValueError("No useful combination found")
        
        steps = []
        
        # 显示原始表达式
        original_expr = " × ".join(map(str, values))
        steps.append(CalculationStep(
            description=f"{original_expr} 使用交换结合法",
            operation="识别交换结合法",
            result="重新排列因子顺序"
        ))
        
        if len(combination) == 6:  # 两个数的组合
            i, j, val1, val2, product, remaining = combination
            
            steps.append(CalculationStep(
                description=f"识别有用组合：{val1} × {val2} = {product}",
                operation="识别组合",
                result=f"{val1} 和 {val2} 先相乘得到 {product}"
            ))
            
            steps.append(CalculationStep(
                description=f"先计算：{val1} × {val2} = {product}",
                operation="优先计算",
                result=product
            ))
            
            if remaining:
                remaining_expr = " × ".join(map(str, remaining))
                new_expr = f"{product} × {remaining_expr}"
                steps.append(CalculationStep(
                    description=f"重排后：{new_expr}",
                    operation="重新排列",
                    result=f"转换为 {new_expr}"
                ))
                
                # 计算最终结果
                final_result = product
                for val in remaining:
                    final_result *= val
                
                # 逐步计算剩余部分
                current_result = product
                for val in remaining:
                    new_result = current_result * val
                    steps.append(CalculationStep(
                        description=f"{current_result} × {val} = {new_result}",
                        operation="继续计算",
                        result=new_result
                    ))
                    current_result = new_result
                
                final_result = current_result
            else:
                final_result = product
        
        else:  # 三个数的组合 
            i, j, k, val1, val2, val3, intermediate, remaining = combination
            
            steps.append(CalculationStep(
                description=f"识别有用组合：{val1} × {val2} = {intermediate}",
                operation="识别组合",
                result=f"{val1} 和 {val2} 先相乘"
            ))
            
            steps.append(CalculationStep(
                description=f"先计算：{val1} × {val2} = {intermediate}",
                operation="优先计算",
                result=intermediate
            ))
            
            steps.append(CalculationStep(
                description=f"再计算：{intermediate} × {val3} = {intermediate * val3}",
                operation="继续计算",
                result=intermediate * val3
            ))
            
            current_result = intermediate * val3
            
            # 计算剩余因子
            for val in remaining:
                new_result = current_result * val
                steps.append(CalculationStep(
                    description=f"{current_result} × {val} = {new_result}",
                    operation="继续计算",
                    result=new_result
                ))
                current_result = new_result
            
            final_result = current_result
        
        steps.append(CalculationStep(
            description=f"最终结果：{final_result}",
            operation="确认结果",
            result=final_result,
            formula="交换结合法：a×b×c = (a×b)×c，优先计算便于运算的组合"
        ))
        
        return steps


"""
乘以15实现
乘以15，等于原数加它的一半，再乘以10
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class MultiplyByFifteen(MathCalculator):
    """乘以15算法"""
    
    def __init__(self):
        super().__init__("乘15速算", "乘以15等于原数加一半再乘10", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：任意数乘以15"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(a, (int, float)) and isinstance(b, (int, float))):
                return False
            
            # 检查是否有一个数是15
            return a == 15 or b == 15
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建乘15速算步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个是15，哪个是被乘数
        if a == 15:
            multiplied_num = b
        else:
            multiplied_num = a
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{multiplied_num} × 15 使用乘15速算法",
            operation="识别模式",
            result="15 = 10 + 5，利用15 = 1.5 × 10的特点"
        ))
        
        # 计算一半
        half_value = multiplied_num / 2
        steps.append(CalculationStep(
            description=f"{multiplied_num} ÷ 2 = {half_value}",
            operation="计算一半",
            result=half_value
        ))
        
        # 原数加一半
        sum_value = multiplied_num + half_value
        steps.append(CalculationStep(
            description=f"{multiplied_num} + {half_value} = {sum_value}",
            operation="原数加一半",
            result=sum_value
        ))
        
        # 乘以10得到最终结果
        final_result = sum_value * 10
        if final_result.is_integer():
            final_result = int(final_result)
        
        steps.append(CalculationStep(
            description=f"{sum_value} × 10 = {final_result}",
            operation="乘以10",
            result=final_result,
            formula="a × 15 = (a + a÷2) × 10"
        ))
        
        return steps


"""
凑十法实现
20以内进位加法，将其中一个加数拆分成与另一个加数凑成10的数
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class TenComplementAddition(MathCalculator):
    """凑十法（20以内进位加法）"""
    
    def __init__(self):
        super().__init__("凑十法", "20以内进位加法，将其中一个加数拆分成与另一个加数凑成10的数", priority=3)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：两个20以内的数相加且需要进位"""
        if formula.type != "addition":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            a, b = [elem.get_numeric_value() for elem in numbers]
            # 20以内的整数，且相加需要进位（结果>10）
            return (isinstance(a, int) and isinstance(b, int) and 
                   0 < a <= 20 and 0 < b <= 20 and a + b > 10 and a + b <= 20)
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建凑十法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 确定哪个数更接近10
        if a >= b:
            larger, smaller = a, b
        else:
            larger, smaller = b, a
        
        # 计算需要凑十的数
        complement = 10 - larger
        remainder = smaller - complement
        
        steps = [
            CalculationStep(
                description=f"观察发现 {larger} 和 {complement} 可以凑成 10",
                operation=f"寻找凑十数",
                result=f"{larger} + {complement} = 10"
            ),
            CalculationStep(
                description=f"将 {smaller} 拆分为 {complement} 和 {remainder}",
                operation=f"拆分小数",
                result=f"{smaller} = {complement} + {remainder}"
            ),
            CalculationStep(
                description=f"先算 {larger} + {complement} = 10",
                operation=f"凑十计算",
                result=10
            ),
            CalculationStep(
                description=f"再算 10 + {remainder} = {10 + remainder}",
                operation=f"加剩余数",
                result=10 + remainder
            )
        ]
        
        return steps


"""
分拆法减法实现
将被减数或减数拆分成更易计算的部分，再进行运算
避免借位，简化计算过程
"""

from typing import List
from ..base_types import MathCalculator, Formula, CalculationStep


class SplitSubtraction(MathCalculator):
    """分拆法减法算法"""
    
    def __init__(self):
        super().__init__("分拆法减法", "将数字拆分成便于计算的部分进行减法", priority=5)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：可以通过拆分简化的减法"""
        if formula.type != "subtraction":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            minuend, subtrahend = [elem.get_numeric_value() for elem in numbers]
            if not (isinstance(minuend, (int, float)) and isinstance(subtrahend, (int, float))):
                return False
            
            # 转换为整数
            minuend, subtrahend = int(minuend), int(subtrahend)
            
            # 只处理正数且被减数大于减数
            if minuend <= subtrahend or minuend < 10 or subtrahend < 10:
                return False
            
            # 检查是否有合适的拆分方式
            return self._find_best_split(minuend, subtrahend) is not None
            
        except:
            return False
    
    def _find_best_split(self, minuend: int, subtrahend: int):
        """寻找最佳拆分方式"""
        candidates = []
        
        # 拆分减数（更常用）
        subtrahend_splits = self._get_useful_splits(subtrahend)
        for split in subtrahend_splits:
            score = self._calculate_split_score(split, minuend, 'subtrahend')
            if score > 0:
                candidates.append(('subtrahend', split, score))
        
        # 拆分被减数（较少使用）
        minuend_splits = self._get_useful_splits(minuend)
        for split in minuend_splits:
            score = self._calculate_split_score(split, subtrahend, 'minuend')
            if score > 0:
                candidates.append(('minuend', split, score))
        
        if not candidates:
            return None
        
        # 选择得分最高的分拆方案
        best = max(candidates, key=lambda x: x[2])
        return best[0], best[1]
    
    def _get_useful_splits(self, num: int) -> List[tuple]:
        """获取有用的分拆方案，返回(主要部分, 次要部分)的列表"""
        splits = []
        
        # 按十位分拆 (如38 = 30 + 8)
        if num >= 20:
            tens = (num // 10) * 10
            ones = num % 10
            if ones != 0:
                splits.append((tens, ones))
        
        # 按百位分拆 (如238 = 200 + 38)
        if num >= 200:
            hundreds = (num // 100) * 100
            remainder = num % 100
            if remainder != 0:
                splits.append((hundreds, remainder))
        
        # 接近整十数的分拆 (如29 = 30 - 1, 31 = 30 + 1)
        if 11 <= num <= 99:
            lower_ten = (num // 10) * 10
            upper_ten = lower_ten + 10
            
            # 优先选择更接近的整十数
            if abs(num - lower_ten) <= abs(num - upper_ten):
                if num != lower_ten:
                    splits.append((lower_ten, num - lower_ten))
            else:
                splits.append((upper_ten, num - upper_ten))
        
        # 接近整百数的分拆
        if 101 <= num <= 999:
            lower_hundred = (num // 100) * 100
            upper_hundred = lower_hundred + 100
            
            # 优先选择更接近的整百数
            if abs(num - lower_hundred) <= abs(num - upper_hundred):
                if num != lower_hundred:
                    splits.append((lower_hundred, num - lower_hundred))
            else:
                splits.append((upper_hundred, num - upper_hundred))
        
        return splits
    
    def _calculate_split_score(self, split: tuple, other_num: int, split_type: str) -> int:
        """计算分拆方案的得分"""
        main_part, minor_part = split
        score = 0
        
        # 主要部分是整十、整百的倍数加分
        if main_part % 10 == 0:
            score += 3
        if main_part % 100 == 0:
            score += 2
        
        # 次要部分越小越好
        abs_minor = abs(minor_part)
        if abs_minor <= 5:
            score += 2
        elif abs_minor <= 10:
            score += 1
        
        # 检查拆分后是否能避免借位
        if split_type == 'subtrahend':
            # 拆分减数，检查是否避免借位
            if other_num >= main_part:  # 主要部分能减
                score += 2
            if (other_num - main_part) >= minor_part:  # 余数部分也能减
                score += 2
        
        # 避免无意义的分拆
        if abs_minor == 0 or abs_minor >= main_part:
            score = 0
        
        return score
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建分拆法步骤"""
        numbers = formula.get_numbers()
        minuend, subtrahend = [int(elem.get_numeric_value()) for elem in numbers]
        
        # 找到最佳拆分方案
        result = self._find_best_split(minuend, subtrahend)
        if result is None:
            raise ValueError("No suitable split found")
        
        split_type, (main_part, minor_part) = result
        
        steps = []
        
        steps.append(CalculationStep(
            description=f"{minuend} - {subtrahend} 使用分拆法",
            operation="识别分拆法",
            result=f"将{'减数' if split_type == 'subtrahend' else '被减数'}分拆为便于计算的形式"
        ))
        
        if split_type == 'subtrahend':
            # 拆分减数
            if minor_part >= 0:
                steps.append(CalculationStep(
                    description=f"分拆减数：{subtrahend} = {main_part} + {minor_part}",
                    operation="减数分拆",
                    result=f"{subtrahend} = {main_part} + {minor_part}"
                ))
                
                steps.append(CalculationStep(
                    description=f"应用减法性质：{minuend} - ({main_part} + {minor_part}) = ({minuend} - {main_part}) - {minor_part}",
                    operation="减法性质",
                    result="分两步进行减法"
                ))
            else:
                steps.append(CalculationStep(
                    description=f"分拆减数：{subtrahend} = {main_part} - {abs(minor_part)}",
                    operation="减数分拆",
                    result=f"{subtrahend} = {main_part} + ({minor_part})"
                ))
                
                steps.append(CalculationStep(
                    description=f"应用减法性质：{minuend} - ({main_part} + ({minor_part})) = ({minuend} - {main_part}) - ({minor_part})",
                    operation="减法性质",
                    result="分两步进行减法"
                ))
            
            # 第一步计算
            step1_result = minuend - main_part
            steps.append(CalculationStep(
                description=f"第一步：{minuend} - {main_part} = {step1_result}",
                operation="计算第一步",
                result=step1_result
            ))
            
            # 第二步计算
            final_result = step1_result - minor_part
            steps.append(CalculationStep(
                description=f"第二步：{step1_result} - {minor_part} = {final_result}",
                operation="计算第二步",
                result=final_result
            ))
            
        else:
            # 拆分被减数
            if minor_part >= 0:
                steps.append(CalculationStep(
                    description=f"分拆被减数：{minuend} = {main_part} + {minor_part}",
                    operation="被减数分拆", 
                    result=f"{minuend} = {main_part} + {minor_part}"
                ))
                
                steps.append(CalculationStep(
                    description=f"应用减法性质：({main_part} + {minor_part}) - {subtrahend} = ({main_part} - {subtrahend}) + {minor_part}",
                    operation="减法性质",
                    result="分两步进行减法"
                ))
            else:
                steps.append(CalculationStep(
                    description=f"分拆被减数：{minuend} = {main_part} - {abs(minor_part)}",
                    operation="被减数分拆",
                    result=f"{minuend} = {main_part} + ({minor_part})"
                ))
            
            # 计算过程
            step1_result = main_part - subtrahend
            steps.append(CalculationStep(
                description=f"第一步：{main_part} - {subtrahend} = {step1_result}",
                operation="计算第一步",
                result=step1_result
            ))
            
            final_result = step1_result + minor_part
            steps.append(CalculationStep(
                description=f"第二步：{step1_result} + {minor_part} = {final_result}",
                operation="计算第二步",
                result=final_result
            ))
        
        steps.append(CalculationStep(
            description=f"最终结果：{final_result}",
            operation="确认结果",
            result=final_result,
            formula="分拆法：将复杂减法拆分为简单减法的组合"
        ))
        
        return steps


"""
小数分配律乘法算子
拆分为整数部分和小数部分相乘再加
"""

from typing import List
from .decima_calculator import DecimaCalculator
from ..base_types import Formula, CalculationStep


class DecimalDistributionMultiply(DecimaCalculator):
    """小数分配律乘法算子"""
    
    def __init__(self):
        super().__init__("小数分配律乘法", "拆分为整数部分和小数部分相乘再加", priority=6)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：小数乘法且至少有一个数可以拆分为整数+小数"""
        if not super().is_match_pattern(formula):
            return False
        
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        # 检查是否有数字可以方便地拆分（整数部分较大，小数部分简单）
        return self._can_split_conveniently(a) or self._can_split_conveniently(b)
    
    def _can_split_conveniently(self, num: float) -> bool:
        """判断数字是否适合用分配律拆分"""
        if isinstance(num, int):
            return False
        
        integer_part = int(num)
        decimal_part = num - integer_part
        
        # 整数部分>=1且小数部分简单（如0.1, 0.2, 0.5等）
        return integer_part >= 1 and decimal_part in [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建分配律乘法步骤"""
        numbers = formula.get_numbers()
        a, b = [elem.get_numeric_value() for elem in numbers]
        
        steps = []
        
        # 确定哪个数适合拆分
        if self._can_split_conveniently(a):
            split_num, other_num = a, b
            split_name, other_name = "第一个数", "第二个数"
        else:
            split_num, other_num = b, a
            split_name, other_name = "第二个数", "第一个数"
        
        # 拆分数字
        integer_part = int(split_num)
        decimal_part = split_num - integer_part
        
        steps.append(CalculationStep(
            description=f"拆分{split_name}：{split_num} = {integer_part} + {decimal_part}",
            operation="拆分数字",
            result=f"{integer_part} + {decimal_part}"
        ))
        
        # 应用分配律
        steps.append(CalculationStep(
            description=f"应用分配律：{split_num} × {other_num} = ({integer_part} + {decimal_part}) × {other_num}",
            operation="应用分配律",
            result=f"{integer_part} × {other_num} + {decimal_part} × {other_num}"
        ))
        
        # 计算整数部分乘积
        integer_product = integer_part * other_num
        steps.append(CalculationStep(
            description=f"整数部分：{integer_part} × {other_num} = {integer_product}",
            operation="计算整数部分",
            result=integer_product
        ))
        
        # 计算小数部分乘积
        decimal_product = decimal_part * other_num
        steps.append(CalculationStep(
            description=f"小数部分：{decimal_part} × {other_num} = {decimal_product}",
            operation="计算小数部分",
            result=decimal_product
        ))
        
        # 相加得到最终结果
        final_result = integer_product + decimal_product
        steps.append(CalculationStep(
            description=f"相加：{integer_product} + {decimal_product} = {final_result}",
            operation="合并结果",
            result=final_result,
            formula="分配律：(a+b)×c = a×c + b×c"
        ))
        
        return steps



"""
分数乘法算子 - 分数×分数
"""

from typing import Dict, Any
from fractions import Fraction
from math import gcd
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType, FractionType


class FractionMultiplyCalculator(FractionCalculator):
    """分数乘法算子"""
    
    def __init__(self):
        super().__init__("分数乘法", "分数相乘，优先交叉约分", priority=5)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配分数乘法问题"""
        if problem.operation != OperationType.MULTIPLY:
            return {"matched": False, "score": 0.0, "reason": "不是乘法"}
        
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        
        # 检查是否都是分数
        has_fraction = any(op.fraction_type in [FractionType.PROPER, FractionType.IMPROPER, FractionType.MIXED] 
                          for op in problem.operands)
        
        if has_fraction:
            return {"matched": True, "score": 0.9, "reason": "分数乘法"}
        else:
            return {"matched": False, "score": 0.0, "reason": "不包含分数"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行分数乘法"""
        try:
            operand1, operand2 = problem.operands
            steps = []
            
            # 步骤1：将带分数转为假分数
            frac1 = operand1.fraction
            frac2 = operand2.fraction
            
            if operand1.is_mixed:
                improper1 = Fraction(operand1.whole_part * frac1.denominator + frac1.numerator, frac1.denominator)
                steps.append(FractionStep(
                    description=f"带分数转假分数：{operand1.raw} → {improper1}",
                    operation="转为假分数",
                    result=str(improper1)
                ))
                frac1 = improper1
            
            if operand2.is_mixed:
                improper2 = Fraction(operand2.whole_part * frac2.denominator + frac2.numerator, frac2.denominator)
                steps.append(FractionStep(
                    description=f"带分数转假分数：{operand2.raw} → {improper2}",
                    operation="转为假分数", 
                    result=str(improper2)
                ))
                frac2 = improper2
            
            # 步骤2：交叉约分
            num1, den1 = frac1.numerator, frac1.denominator
            num2, den2 = frac2.numerator, frac2.denominator
            
            # 约分 num1 与 den2
            gcd1 = gcd(abs(num1), abs(den2))
            if gcd1 > 1:
                num1 //= gcd1
                den2 //= gcd1
                steps.append(FractionStep(
                    description=f"交叉约分：分子{frac1.numerator}与分母{frac2.denominator}的公因子{gcd1}",
                    operation="交叉约分",
                    result=f"约分后：{num1}，{den2}"
                ))
            
            # 约分 num2 与 den1
            gcd2 = gcd(abs(num2), abs(den1))
            if gcd2 > 1:
                num2 //= gcd2
                den1 //= gcd2
                steps.append(FractionStep(
                    description=f"交叉约分：分子{frac2.numerator}与分母{frac1.denominator}的公因子{gcd2}",
                    operation="交叉约分",
                    result=f"约分后：{num2}，{den1}"
                ))
            
            # 步骤3：分子相乘，分母相乘
            result_num = num1 * num2
            result_den = den1 * den2
            result_fraction = Fraction(result_num, result_den)
            
            steps.append(FractionStep(
                description=f"分子相乘，分母相乘：#frac{{({num1}×{num2})}}{{({den1}×{den2})}}# = #frac{{{result_num}}}{{{result_den}}}#",
                operation="分子分母分别相乘",
                result=str(result_fraction),
                formula="#frac{a}{b}#×#frac{c}{d}# = #frac{a×c}{b×d}#"
            ))
            
            # 步骤4：最终约分
            final_result = result_fraction
            if result_fraction.denominator != 1:
                final_gcd = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if final_gcd > 1:
                    final_result = Fraction(result_fraction.numerator // final_gcd,
                                          result_fraction.denominator // final_gcd)
                    steps.append(FractionStep(
                        description=f"最终约分：{result_fraction} → {final_result}",
                        operation="取得最简分数",
                        result=str(final_result)
                    ))
            
            # 步骤5：根据需要转为带分数
            display_result = final_result
            if (problem.display_mixed and 
                abs(final_result.numerator) >= final_result.denominator and 
                final_result.denominator != 1):
                
                whole = final_result.numerator // final_result.denominator
                remainder = abs(final_result.numerator) % final_result.denominator
                
                if remainder == 0:
                    display_result = str(whole)
                else:
                    display_result = f"{whole}#frac{{{remainder}}}{{{final_result.denominator}}}#"
                
                steps.append(FractionStep(
                    description=f"转为带分数：{final_result} = {display_result}",
                    operation="转为带分数",
                    result=str(display_result)
                ))
            
            return FractionResult(
                success=True,
                result=display_result,
                steps=steps,
                step_count=len(steps),
                formula=f"分数乘法：{frac1}×{frac2} = {display_result}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"分数乘法计算失败: {str(e)}"
            )



"""
一般乘法实现
支持逐位相乘法的通用乘法算法，包括特殊规则处理、负数和小数支持
"""

from typing import List, Union
from decimal import Decimal, getcontext
from fractions import Fraction
from ..base_types import MathCalculator, Formula, CalculationStep

# 设置小数精度
getcontext().prec = 28


class GeneralMultiplication(MathCalculator):
    """一般乘法算法（逐位相乘法）"""
    
    def __init__(self):
        super().__init__("一般乘法", "通用乘法", priority=1)
    
    def is_match_pattern(self, formula: Formula) -> bool:
        """匹配模式：所有乘法运算（最低优先级，兜底算法）"""
        if formula.type != "multiplication":
            return False
        
        numbers = formula.get_numbers()
        if len(numbers) != 2:
            return False
        
        try:
            factor1, factor2 = [elem.get_numeric_value() for elem in numbers]
            # 支持整数、浮点数
            return isinstance(factor1, (int, float)) and isinstance(factor2, (int, float))
        except:
            return False
    
    def construct_steps(self, formula: Formula) -> List[CalculationStep]:
        """构建一般乘法步骤"""
        numbers = formula.get_numbers()
        factor1 = numbers[0].get_numeric_value()
        factor2 = numbers[1].get_numeric_value()
        
        # 转换Fraction为float以便处理
        if isinstance(factor1, Fraction):
            factor1 = float(factor1)
        if isinstance(factor2, Fraction):
            factor2 = float(factor2)
        
        # 检查特殊情况（按优先级顺序）
        if factor1 == 0 or factor2 == 0:
            self.name = "零乘法"
            self.priority = 9
            return self._construct_zero_multiplication_steps(factor1, factor2)
        elif factor1 == 1 or factor2 == 1:
            self.name = "一乘法"
            self.priority = 9
            return self._construct_one_multiplication_steps(factor1, factor2)
        elif self._is_single_digit_multiplication(factor1, factor2):
            self.name = "表内乘法"
            self.priority = 9
            return self._construct_single_digit_steps(factor1, factor2)
        elif isinstance(factor1, float) or isinstance(factor2, float):
            return self._construct_decimal_steps(factor1, factor2)
        elif factor1 < 0 or factor2 < 0:
            return self._construct_negative_steps(factor1, factor2)
        else:
            return self._construct_general_steps(int(factor1), int(factor2))
    
    def _is_single_digit_multiplication(self, factor1: Union[int, float], factor2: Union[int, float]) -> bool:
        """判断是否为两个个位数的乘法"""
        return (isinstance(factor1, int) and isinstance(factor2, int) and 
                1 <= factor1 <= 9 and 1 <= factor2 <= 9)
    
    def _construct_zero_multiplication_steps(self, factor1: Union[int, float], factor2: Union[int, float]) -> List[CalculationStep]:
        """构建0乘法的特殊步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} = 0",
            operation="0乘法规律",
            result=0,
            formula="0与任何数相乘都等于0"
        ))
        
        return steps
    
    def _construct_one_multiplication_steps(self, factor1: Union[int, float], factor2: Union[int, float]) -> List[CalculationStep]:
        """构建1乘法的特殊步骤"""
        steps = []
        
        result = factor2 if factor1 == 1 else factor1
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} = {result}",
            operation="1乘法规律",
            result=result,
            formula="1与任何数相乘都等于原数本身"
        ))
        
        return steps
    
    def _construct_single_digit_steps(self, factor1: int, factor2: int) -> List[CalculationStep]:
        """构建个位数乘法步骤（使用乘法口诀）"""
        steps = []
        result = factor1 * factor2
        
        # 构建乘法口诀
        multiplication_table = {
            (1, 1): "一一得一", (1, 2): "一二得二", (1, 3): "一三得三", (1, 4): "一四得四", 
            (1, 5): "一五得五", (1, 6): "一六得六", (1, 7): "一七得七", (1, 8): "一八得八", (1, 9): "一九得九",
            (2, 2): "二二得四", (2, 3): "二三得六", (2, 4): "二四得八", (2, 5): "二五一十", 
            (2, 6): "二六十二", (2, 7): "二七74", (2, 8): "二八十六", (2, 9): "二九十八",
            (3, 3): "三三得九", (3, 4): "三四十二", (3, 5): "三五55", (3, 6): "三六68", 
            (3, 7): "三七二十一", (3, 8): "三八二十四", (3, 9): "三九二十七",
            (4, 4): "四四十六", (4, 5): "四五二十", (4, 6): "四六二十四", (4, 7): "四七二十八", 
            (4, 8): "四八三十二", (4, 9): "四九三十六",
            (5, 5): "五五二十五", (5, 6): "五六三十", (5, 7): "五七三十五", (5, 8): "五八四十", (5, 9): "五九四十五",
            (6, 6): "六六三十六", (6, 7): "六七四十二", (6, 8): "六八四十八", (6, 9): "六九五十四",
            (7, 7): "七七四十九", (7, 8): "七八五十六", (7, 9): "七九六十三",
            (8, 8): "八八六十四", (8, 9): "八九七十二",
            (9, 9): "九九八十一"
        }
        
        # 获取口诀（确保顺序正确）
        key = (min(factor1, factor2), max(factor1, factor2))
        formula_text = multiplication_table.get(key, f"{factor1}×{factor2}={result}")
        
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} = {result}",
            operation="表内乘法口诀",
            result=result,
            formula=formula_text
        ))
        
        return steps
        
    def _construct_negative_steps(self, factor1: Union[int, float], factor2: Union[int, float]) -> List[CalculationStep]:
        """构建负数乘法步骤"""
        steps = []
        
        # 判断符号
        is_factor1_negative = factor1 < 0
        is_factor2_negative = factor2 < 0
        result_positive = not (is_factor1_negative ^ is_factor2_negative)  # 异或取反
        
        abs_factor1 = abs(factor1)
        abs_factor2 = abs(factor2)
        
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} 处理负数乘法",
            operation="分析符号",
            result="负数乘法：同号为正，异号为负"
        ))
        
        steps.append(CalculationStep(
            description=f"符号分析：{factor1} {'< 0' if is_factor1_negative else '> 0'}, {factor2} {'< 0' if is_factor2_negative else '> 0'}",
            operation="确定结果符号",
            result=f"结果为{'正数' if result_positive else '负数'}"
        ))
        
        steps.append(CalculationStep(
            description=f"取绝对值：|{factor1}| = {abs_factor1}, |{factor2}| = {abs_factor2}",
            operation="计算绝对值",
            result=f"计算 {abs_factor1} × {abs_factor2}"
        ))
        
        # 递归处理绝对值乘法
        abs_result = abs_factor1 * abs_factor2
        
        if isinstance(abs_factor1, int) and isinstance(abs_factor2, int):
            # 整数情况，展示逐位相乘
            steps.extend(self._construct_digit_multiplication_steps(abs_factor1, abs_factor2, int(abs_result)))
        else:
            # 小数情况，简化处理
            steps.append(CalculationStep(
                description=f"{abs_factor1} × {abs_factor2} = {abs_result}",
                operation="计算绝对值乘积",
                result=abs_result
            ))
        
        final_result = abs_result if result_positive else -abs_result
        steps.append(CalculationStep(
            description=f"添加符号：{abs_result} → {final_result}",
            operation="确定最终结果",
            result=final_result,
            formula="负数乘法：(-a) × (-b) = ab, (-a) × b = -ab"
        ))
        
        return steps
    
    def _construct_decimal_steps(self, factor1: Union[int, float], factor2: Union[int, float]) -> List[CalculationStep]:
        """构建小数乘法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} 处理小数乘法",
            operation="识别小数",
            result="小数乘法：先按整数相乘，再确定小数点位置"
        ))
        
        # 转换为字符串以便处理小数点
        str1 = str(factor1)
        str2 = str(factor2)
        
        decimal_places1 = len(str1.split('.')[1]) if '.' in str1 else 0
        decimal_places2 = len(str2.split('.')[1]) if '.' in str2 else 0
        total_decimal_places = decimal_places1 + decimal_places2
        
        steps.append(CalculationStep(
            description=f"小数位数：{factor1} 有 {decimal_places1} 位小数，{factor2} 有 {decimal_places2} 位小数",
            operation="统计小数位数",
            result=f"结果应有 {total_decimal_places} 位小数"
        ))
        
        # 转换为整数计算
        multiplier1 = 10 ** decimal_places1
        multiplier2 = 10 ** decimal_places2
        int_factor1 = int(factor1 * multiplier1)
        int_factor2 = int(factor2 * multiplier2)
        
        if decimal_places1 > 0 or decimal_places2 > 0:
            steps.append(CalculationStep(
                description=f"转换为整数：{factor1} × {multiplier1} = {int_factor1}, {factor2} × {multiplier2} = {int_factor2}",
                operation="消除小数点",
                result=f"计算 {int_factor1} × {int_factor2}"
            ))
        
        int_result = int_factor1 * int_factor2
        
        # 如果是小整数，展示逐位相乘
        if int_factor1 < 1000 and int_factor2 < 1000:
            steps.extend(self._construct_digit_multiplication_steps(int_factor1, int_factor2, int_result))
        else:
            steps.append(CalculationStep(
                description=f"{int_factor1} × {int_factor2} = {int_result}",
                operation="整数相乘",
                result=int_result
            ))
        
        final_result = factor1 * factor2
        if total_decimal_places > 0:
            steps.append(CalculationStep(
                description=f"确定小数点位置：{int_result} ÷ {10**total_decimal_places} = {final_result}",
                operation="还原小数点",
                result=final_result,
                formula="小数乘法：结果小数位数 = 两因数小数位数之和"
            ))
        else:
            steps.append(CalculationStep(
                description=f"最终结果：{final_result}",
                operation="确定结果",
                result=final_result
            ))
        
        return steps
    
    def _construct_general_steps(self, factor1: int, factor2: int) -> List[CalculationStep]:
        """构建一般整数乘法步骤"""
        steps = []
        
        steps.append(CalculationStep(
            description=f"{factor1} × {factor2} 使用逐位相乘法",
            operation="识别一般乘法",
            result="按照位数逐位相乘，再按权相加"
        ))
        
        result = factor1 * factor2
        steps.extend(self._construct_digit_multiplication_steps(factor1, factor2, result))
        
        return steps
    
    def _construct_digit_multiplication_steps(self, factor1: int, factor2: int, expected_result: int) -> List[CalculationStep]:
        """构建逐位相乘的详细步骤"""
        steps = []
        
        # 将因数按位分解
        str1 = str(abs(factor1))
        str2 = str(abs(factor2))
        
        # 选择较小的数作为乘数，较大的数作为被乘数（提高效率）
        if len(str2) < len(str1):
            factor1, factor2 = factor2, factor1
            str1, str2 = str2, str1
        
        steps.append(CalculationStep(
            description=f"分解乘数：{factor2} 按位分解进行逐位相乘",
            operation="分解乘数",
            result=f"以 {factor1} 为被乘数，{factor2} 为乘数"
        ))
        
        partial_products = []
        
        # 逐位相乘
        for i, digit2 in enumerate(reversed(str2)):
            digit_val = int(digit2)
            position_value = 10 ** i
            
            if digit_val == 0:
                partial_product = 0
                steps.append(CalculationStep(
                    description=f"{factor1} × {digit_val} × {position_value} = 0（个位为0）",
                    operation=f"第{i+1}位相乘",
                    result=0
                ))
            else:
                basic_product = factor1 * digit_val
                partial_product = basic_product * position_value
                
                steps.append(CalculationStep(
                    description=f"{factor1} × {digit_val}（{position_value}位）= {factor1} × {digit_val} × {position_value} = {partial_product}",
                    operation=f"第{i+1}位相乘",
                    result=partial_product
                ))
            
            partial_products.append(partial_product)
        
        # 显示部分积
        if len(partial_products) > 1:
            non_zero_products = [p for p in partial_products if p != 0]
            steps.append(CalculationStep(
                description=f"部分积：{' + '.join(map(str, non_zero_products))}",
                operation="列出所有部分积",
                result=f"需要将这些部分积相加"
            ))
            
            # 逐步相加
            running_sum = 0
            for i, product in enumerate(partial_products):
                if product != 0:
                    running_sum += product
                    if i == 0:
                        steps.append(CalculationStep(
                            description=f"累加：{product}",
                            operation="开始累加",
                            result=running_sum
                        ))
                    else:
                        steps.append(CalculationStep(
                            description=f"累加：{running_sum - product} + {product} = {running_sum}",
                            operation="继续累加",
                            result=running_sum
                        ))
        
        final_result = sum(partial_products)
        steps.append(CalculationStep(
            description=f"最终结果：{factor1} × {factor2} = {final_result}",
            operation="完成计算",
            result=final_result,
            formula="逐位相乘法：(AB) × C = A×10×C + B×C"
        ))
        
        return steps


"""
Calculator package
包含所有具体的算子实现
"""

from .multiply_by_five import MultiplyByFive
from .division_with_remainder import DivisionWithRemainder
from .same_head_sum_ten_multiplication import SameHeadSumTenMultiplication
from .head_sum_ten_tail_same_multiplication import HeadSumTenTailSameMultiplication
from .ten_complement_addition import TenComplementAddition
from .break_ten_subtraction import BreakTenSubtraction
from .reference_number_addition import ReferenceNumberAddition
from .high_to_low_digit_addition import HighToLowDigitAddition
from .segment_sum_addition import SegmentSumAddition
from .rounding_addition import RoundingAddition
from .split_addition import SplitAddition
from .group_pairing_addition import GroupPairingAddition
from .borrow_rounding_addition import BorrowRoundingAddition
from .commutative_associative_addition import CommutativeAssociativeAddition
from .multiply_by_powers_of_ten import MultiplyByPowersOfTen
from .multiply_by_one_hundred_twenty_five import MultiplyByOneHundredTwentyFive
from .multiply_by_twelve import MultiplyByTwelve
from .multiply_by_fifteen import MultiplyByFifteen
from .multiply_by_nineteen import MultiplyByNineteen
from .difference_method_multiplication import DifferenceMethodMultiplication
from .split_method_multiplication import SplitMethodMultiplication
from .commutative_associative_multiplication import CommutativeAssociativeMultiplication

__all__ = [
    'DivisionWithRemainder',
    'MultiplyByFive',
    'SameHeadSumTenMultiplication',
    'HeadSumTenTailSameMultiplication',
    'TenComplementAddition',
    'BreakTenSubtraction',
    'ReferenceNumberAddition',
    'HighToLowDigitAddition',
    'SegmentSumAddition',
    'RoundingAddition',
    'SplitAddition',
    'GroupPairingAddition',
    'BorrowRoundingAddition',
    'CommutativeAssociativeAddition',
    'MultiplyByPowersOfTen',
    'MultiplyByOneHundredTwentyFive',
    'MultiplyByTwelve',
    'MultiplyByFifteen',
    'MultiplyByNineteen',
    'DifferenceMethodMultiplication',
    'SplitMethodMultiplication',
    'CommutativeAssociativeMultiplication'
]


"""
同分母分数减法算子
"""

from typing import Dict, Any
from fractions import Fraction
from math import gcd
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType


class SameDenominatorSubtractCalculator(FractionCalculator):
    """同分母分数减法算子"""
    
    def __init__(self):
        super().__init__("同分母减法", "分母相同的分数直接相减分子", priority=6)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配同分母减法问题"""
        if problem.operation != OperationType.SUBTRACT:
            return {"matched": False, "score": 0.0, "reason": "不是减法"}
        
        if len(problem.operands) != 2:
            return {"matched": False, "score": 0.0, "reason": "需要两个操作数"}
        
        # 检查是否同分母
        frac1, frac2 = problem.operands[0].fraction, problem.operands[1].fraction
        same_denom = frac1.denominator == frac2.denominator
        
        if same_denom:
            return {"matched": True, "score": 1.0, "reason": "同分母减法"}
        else:
            return {"matched": False, "score": 0.0, "reason": "分母不同"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行同分母减法"""
        try:
            operand1, operand2 = problem.operands
            frac1, frac2 = operand1.fraction, operand2.fraction
            steps = []
            
            # 步骤1：确认同分母
            common_denom = frac1.denominator
            
            steps.append(FractionStep(
                description=f"确认分母相同：{frac1} - {frac2}，分母都是{common_denom}",
                operation="确认分母相同",
                result=f"公分母：{common_denom}"
            ))
            
            # 步骤2：分子相减
            diff_numerator = frac1.numerator - frac2.numerator
            result_fraction = Fraction(diff_numerator, common_denom)
            
            steps.append(FractionStep(
                description=f"分子相减：{frac1.numerator} - {frac2.numerator} = {diff_numerator}",
                operation="分子相减，分母不变",
                result=f"{diff_numerator}/{common_denom}",
                formula=f"结果 = {diff_numerator}/{common_denom}"
            ))
            
            # 步骤3：约分
            reduced_result = result_fraction
            if result_fraction.denominator != 1:
                common_factor = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if common_factor > 1:
                    reduced_result = Fraction(result_fraction.numerator // common_factor,
                                            result_fraction.denominator // common_factor)
                    steps.append(FractionStep(
                        description=f"约分：{result_fraction} → {reduced_result}",
                        operation="约分",
                        result=str(reduced_result),
                        formula=f"约去公因子{common_factor}"
                    ))
            
            # 步骤4：根据需要转为带分数
            final_result = reduced_result
            if (problem.display_mixed and 
                abs(reduced_result.numerator) >= reduced_result.denominator and 
                reduced_result.denominator != 1):
                
                whole = reduced_result.numerator // reduced_result.denominator
                remainder = abs(reduced_result.numerator) % reduced_result.denominator
                
                if remainder == 0:
                    final_result = str(whole)
                else:
                    if reduced_result.numerator < 0:
                        final_result = f"-{abs(whole)} {remainder}/{reduced_result.denominator}"
                    else:
                        final_result = f"{whole} {remainder}/{reduced_result.denominator}"
                
                steps.append(FractionStep(
                    description=f"转为带分数：{reduced_result} = {final_result}",
                    operation="转为带分数",
                    result=str(final_result)
                ))
            
            return FractionResult(
                success=True,
                result=final_result,
                steps=steps,
                step_count=len(steps),
                formula=f"同分母减法：{frac1} - {frac2} = {final_result}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"同分母减法计算失败: {str(e)}"
            )



"""
同分母分数加法算子
"""

from typing import Dict, Any
from fractions import Fraction
from ..base_types import FractionCalculator, FractionProblem, FractionResult, FractionStep, OperationType


class SameDenominatorAddCalculator(FractionCalculator):
    """同分母分数加法算子"""
    
    def __init__(self):
        super().__init__("同分母加法", "分母相同的分数直接相加分子", priority=6)
    
    def is_match_pattern(self, problem: FractionProblem) -> Dict[str, Any]:
        """匹配同分母加法问题"""
        if problem.operation != OperationType.ADD:
            return {"matched": False, "score": 0.0, "reason": "不是加法"}
        
        if len(problem.operands) < 2:
            return {"matched": False, "score": 0.0, "reason": "操作数不足"}
        
        # 检查所有分数是否同分母
        first_denom = problem.operands[0].fraction.denominator
        same_denom = all(op.fraction.denominator == first_denom for op in problem.operands)
        
        if same_denom:
            return {"matched": True, "score": 1.0, "reason": "同分母加法"}
        else:
            return {"matched": False, "score": 0.0, "reason": "分母不同"}
    
    def solve(self, problem: FractionProblem) -> FractionResult:
        """执行同分母加法"""
        try:
            operands = problem.operands
            steps = []
            
            # 步骤1：确认同分母
            common_denom = operands[0].fraction.denominator
            fractions = [op.fraction for op in operands]
            
            steps.append(FractionStep(
                description=f"确认分母相同：{' + '.join(str(f) for f in fractions)}，分母都是{common_denom}",
                operation="确认分母相同",
                result=f"公分母：{common_denom}"
            ))
            
            # 步骤2：分子相加
            sum_numerator = sum(f.numerator for f in fractions)
            result_fraction = Fraction(sum_numerator, common_denom)
            
            numerators_str = ' + '.join(str(f.numerator) for f in fractions)
            steps.append(FractionStep(
                description=f"分子相加：{numerators_str} = {sum_numerator}",
                operation="分子相加",
                result=f"{sum_numerator}/{common_denom}",
                formula=f"结果 = {sum_numerator}/{common_denom}"
            ))
            
            # 步骤3：约分并转换格式
            reduced_result = result_fraction
            if result_fraction.denominator != 1:
                # 检查是否可以约分
                from math import gcd
                common_factor = gcd(abs(result_fraction.numerator), result_fraction.denominator)
                if common_factor > 1:
                    reduced_result = Fraction(result_fraction.numerator // common_factor, 
                                            result_fraction.denominator // common_factor)
                    steps.append(FractionStep(
                        description=f"约分：{result_fraction} → {reduced_result}",
                        operation="约分",
                        result=str(reduced_result),
                        formula=f"约去公因子{common_factor}"
                    ))
            
            # 步骤4：根据需要转为带分数
            final_result = reduced_result
            if (problem.display_mixed and 
                abs(reduced_result.numerator) >= reduced_result.denominator and 
                reduced_result.denominator != 1):
                
                whole = reduced_result.numerator // reduced_result.denominator
                remainder = abs(reduced_result.numerator) % reduced_result.denominator
                
                if remainder == 0:
                    final_result = str(whole)
                else:
                    final_result = f"{whole} {remainder}/{reduced_result.denominator}"
                
                steps.append(FractionStep(
                    description=f"转为带分数：{reduced_result} = {final_result}",
                    operation="转为带分数",
                    result=str(final_result)
                ))
            
            return FractionResult(
                success=True,
                result=final_result,
                steps=steps,
                step_count=len(steps),
                formula=f"同分母加法：{' + '.join(str(f) for f in fractions)} = {final_result}",
                validation=True
            )
            
        except Exception as e:
            return FractionResult(
                success=False,
                error=f"同分母加法计算失败: {str(e)}"
            )
